/* logjam - a GTK client for LiveJournal.
 * Copyright (C) 2000-2003 Evan Martin <evan@livejournal.com>
 *
 * vim: tabstop=4 shiftwidth=4 noexpandtab :
 */

#include "config.h"
#include <glib.h>

#ifndef G_OS_WIN32
#include <unistd.h>
#include <sys/types.h> /* for fork/wait (spawning editor) */
#include <sys/wait.h>
#else
#include <io.h> /* for open/close on Win32 */
#endif

#include <libxml/tree.h>
#include <libxml/parser.h>
#include <stdlib.h>
#include <errno.h>

#include "livejournal.h"
#include "conf.h"
#include "util.h"

static Entry* entry_from_user_editor(const char *filename, GError **err);

Entry *
entry_new(void) {
	Entry *entry = g_new0(Entry, 1);
	entry->security = conf.defaultsecurity;
	return entry;
}
Entry *
entry_copy(Entry *e) {
	Entry *newe = entry_new();
	memcpy(newe, e, sizeof(Entry));
	if (e->subject)
		newe->subject = g_strdup(e->subject);
	if (e->event)
		newe->event = g_strdup(e->event);
	if (e->mood)
		newe->mood = g_strdup(e->mood);
	if (e->music)
		newe->music = g_strdup(e->music);
	if (e->pickeyword)
		newe->pickeyword = g_strdup(e->pickeyword);
	return newe;
}
void
entry_free(Entry *e) {
	g_free(e->subject);
	g_free(e->event);
	g_free(e->mood);
	g_free(e->music);
	g_free(e->pickeyword);
	g_free(e);
}

char*
entry_make_summary(Entry *entry) {
	return get_summary(entry->subject, entry->event);
}

void
entry_set_request_fields(Entry *entry, NetRequest *request) {
	struct tm *ptm = &entry->time;

	/* basic information */
	if (entry->itemid)
		net_request_seti(request, "itemid", entry->itemid);

	net_request_copys(request, "subject", entry->subject ? entry->subject : "");
	net_request_copys(request, "event",   entry->event);

	if (!ptm->tm_year) {
		time_t curtime_time_t = time(NULL);
		ptm = localtime(&curtime_time_t);
	}
	net_request_seti( request, "year", ptm->tm_year+1900);
	net_request_seti( request, "mon",  ptm->tm_mon+1);
	net_request_seti( request, "day",  ptm->tm_mday);
	net_request_seti( request, "hour", ptm->tm_hour);
	net_request_seti( request, "min",  ptm->tm_min);

	/* metadata */
/* http://www.livejournal.com/admin/schema/?mode=viewdata&table=logproplist */
	if (entry->mood) {
		int id = mood_id_from_name(entry->mood);
		if (id >= 0) {
			net_request_seti(request, "prop_current_moodid", id);
		} else {
			net_request_copys(request, "prop_current_mood", entry->mood);
		}	
	} else {
		net_request_copys(request, "prop_current_mood", "");
	}

	net_request_copys(request, 
			"prop_current_music", entry->music ? entry->music : "");
	net_request_copys(request, 
			"prop_picture_keyword", entry->pickeyword ? entry->pickeyword : "");
	net_request_seti(request, "prop_opt_preformatted", entry->preformatted);
	net_request_seti(request, "prop_opt_nocomments", entry->comments == COMMENTS_DISABLE);
	net_request_seti(request, "prop_opt_noemail", entry->comments == COMMENTS_NOEMAIL);
	net_request_seti(request, "prop_opt_backdated", entry->backdated);

	security_append_to_request(&entry->security, request);
}

void
entry_load_metadata(Entry *entry, const char *key, const char *value) {
	g_return_if_fail(key != NULL);

	if (strcmp(key, "current_mood") == 0) {
		entry->mood = g_strdup(value);
		verify_utf8(&entry->mood);
	} else if (strcmp(key, "current_moodid") == 0) {
		int moodid;
		char *mood;
		moodid = atoi(value);
		mood = mood_name_from_id(moodid);
		if (mood)
			entry->mood = g_strdup(mood);
		/*else
			lj_warning(NULL, "entry_load_meta: unknown moodid '%s'.", value);*/
	} else if (strcmp(key, "current_music") == 0) {
		entry->music = g_strdup(value);
		verify_utf8(&entry->music);
	} else if (strcmp(key, "picture_keyword") == 0) {
		entry->pickeyword = g_strdup(value);
		verify_utf8(&entry->pickeyword);
	} else if (strcmp(key, "opt_preformatted") == 0) {
		entry->preformatted = (value && value[0] == '1');
	} else if (strcmp(key, "opt_nocomments") == 0) {
		entry->comments = COMMENTS_DISABLE;
	} else if (strcmp(key, "opt_noemail") == 0) {
		entry->comments = COMMENTS_NOEMAIL;
	} else if (strcmp(key, "opt_backdated") == 0) {
		entry->backdated = (value && value[0] == '1');
#ifdef DEBUG
/* we currently don't have a DEBUG define,
 * but this code may be useful in the future. */
	} else {
		lj_warning(NULL, "entry_load_meta: unknown key '%s'.", key);
#endif
	}
}

static xmlNodePtr
addtextchildenc(xmlDocPtr doc, xmlNodePtr parent, char *name, char *val) {
	char *enc;
	xmlNodePtr node;
	enc = xmlEncodeEntitiesReentrant(doc, val);
	node = xmlNewTextChild(parent, NULL, name, val);
	xmlFree(enc);
	return node;
}

#define XML_ENTRY_META_GET(A)                                          \
    if ((!strcmp(cur->name, #A))) {	                                   \
        entry->A = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1); \
    }

#define XML_ENTRY_META_SET(A)                     \
	if (entry->A)                                 \
		addtextchildenc(doc, root, #A, entry->A);

xmlNodePtr
entry_to_xml_node(Entry *entry, xmlDocPtr doc) {
	xmlNodePtr root, node;
	root = xmlNewDocNode(doc, NULL, "entry", NULL);

	/* do these in the order the fields are specified in livejournal.h,
	 * so we don't miss anything. */
	if (entry->itemid) {
		char buf[10];
		g_snprintf(buf, 10, "%d", entry->itemid);
		xmlSetProp(root, "itemid", buf);
	}
	XML_ENTRY_META_SET(subject);
	XML_ENTRY_META_SET(event);
	XML_ENTRY_META_SET(mood);
	XML_ENTRY_META_SET(music);
	XML_ENTRY_META_SET(pickeyword);
	if (entry->preformatted)
		xmlNewChild(root, NULL, "preformatted", NULL);
	if (entry->backdated)
		xmlNewChild(root, NULL, "backdated", NULL);
	if (entry->comments) {
		char *type = NULL;
		node = xmlNewChild(root, NULL, "comments", NULL);
		switch (entry->comments) {
			case COMMENTS_DEFAULT: break;
			case COMMENTS_NOEMAIL: type = "noemail"; break;
			case COMMENTS_DISABLE: type = "disable"; break;
		}
		xmlSetProp(node, "type", type);
	}
	if (entry->time.tm_year) {
		char *ljdate = tm_to_ljdate(&entry->time);
		addtextchildenc(doc, root, "time", ljdate);
		g_free(ljdate);
	}
	if (entry->security.type != SECURITY_PUBLIC) {
		xmlNodePtr node;
		char *type = NULL, *mask = NULL;
		security_save_to_strings(&entry->security, &type, &mask);
		if (type) {
			node = xmlNewChild(root, NULL, "security", NULL);
			xmlSetProp(node, "type", type);
			if (mask) {
				xmlSetProp(node, "allowmask", mask);
				g_free(mask);
			}
			g_free(type);
		}
	}

	return root;
}

static xmlDocPtr
entry_to_xml(Entry *entry) {
	xmlDocPtr doc;
	xmlNodePtr root;

	doc = xmlNewDoc("1.0");
	root = entry_to_xml_node(entry, doc);
	xmlDocSetRootElement(doc, root);

	return doc;
}

Entry*
entry_new_from_stdin(GError **err) {
	GString *str = g_string_new(NULL);
	char buf[1024];
	int len;
	Entry *entry;

	while ((len = fread(buf, 1, 1024, stdin)) > 0)
		g_string_append_len(str, buf, len);

	entry = entry_new();
	entry->event = str->str;
	g_string_free(str, FALSE);
	return entry;
}

Entry*
entry_new_from_result(NetResult *result, int i) {
	Entry *entry;
	char buf[30];
	char *itemid;

	sprintf(buf, "events_%d_", i);

	itemid = net_result_get_prefix(result, buf, "itemid");
	if (!itemid)
		return NULL;

	entry = entry_new();
	entry->itemid = atoi(itemid);
	ljdate_to_tm(net_result_get_prefix(result, buf, "eventtime"), &entry->time);
	entry->event = urldecode(net_result_get_prefix(result, buf, "event"));
	verify_utf8(&entry->event);
	entry->subject = g_strdup(net_result_get_prefix(result, buf, "subject"));
	verify_utf8(&entry->subject);
	security_load_from_strings(&entry->security, 
			net_result_get_prefix(result, buf, "security"),
			net_result_get_prefix(result, buf, "allowmask"));

	return entry;
}

Entry *
entry_new_single_from_result(NetResult *result) {
	Entry *entry;
	int i, propcount;
	char buf[30];
	char *pname, *pvalue;

	entry = entry_new_from_result(result, 1);

	/* if we retrieved more than one entry, this is definitely wrong. */
	propcount = net_result_geti(result, "prop_count");
	for (i = 1; i <= propcount; i++) {
		sprintf(buf, "prop_%d_", i);
		pname = net_result_get_prefix(result,  buf, "name");
		pvalue = net_result_get_prefix(result, buf, "value");
		entry_load_metadata(entry, pname, pvalue);
	}
	return entry;
}

GHashTable*
entries_new_from_result(NetResult *result, int count, Entry **plastentry) {
	GHashTable *entries;
	Entry *entry = NULL;
	int i, itemid;
	char buf[30];
	char *pname, *pvalue;
	Entry *lastentry = NULL;

	entries = g_hash_table_new(g_int_hash, g_int_equal);

	for (i = 1; i <= count; i++) {
		entry = entry_new_from_result(result, i);
		g_hash_table_insert(entries, &entry->itemid, entry);
		lastentry = entry;
	}
	if (plastentry) *plastentry = entry;

	count = net_result_geti(result, "prop_count");
	for (i = 1; i <= count; i++) {
		sprintf(buf, "prop_%d_", i);
		itemid = atoi(net_result_get_prefix(result, buf, "itemid"));
		pname = net_result_get_prefix(result,  buf, "name");
		pvalue = net_result_get_prefix(result, buf, "value");
		entry = g_hash_table_lookup(entries, &itemid);
		entry_load_metadata(entry, pname, pvalue);
	}

	return entries;
}

static void
entry_load_from_xml_node(Entry *entry, xmlDocPtr doc, xmlNodePtr node) {
	char *itemid;
	xmlNodePtr cur;

	if ((itemid = xmlGetProp(node, "itemid")) != NULL) {
		entry->itemid = atoi(itemid);
		xmlFree(itemid);
	}
	cur = node->xmlChildrenNode;
	while (cur != NULL) {
		XML_ENTRY_META_GET(subject)
		else XML_ENTRY_META_GET(event)
		else XML_ENTRY_META_GET(mood)
		else XML_ENTRY_META_GET(music)
		else XML_ENTRY_META_GET(pickeyword)
		else if (strcmp(cur->name, "preformatted") == 0) {
			entry->preformatted = TRUE;
		} else if (strcmp(cur->name, "backdated") == 0) {
			entry->backdated = TRUE;
		} else if (strcmp(cur->name, "comments") == 0) {
			char *type;
			if ((type = xmlGetProp(cur, "type")) != NULL) {
				if (strcmp(type, "noemail") == 0) {
					entry->comments = COMMENTS_NOEMAIL;
				} else if (strcmp(type, "disable") == 0) {
					entry->comments = COMMENTS_DISABLE;
				}
				xmlFree(type);
			}
		} else if (strcmp(cur->name, "time") == 0) {
			char *date = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			ljdate_to_tm(date, &entry->time);
			g_free(date);
		} else if (strcmp(cur->name, "security") == 0) {
			char *type, *mask;
			type = xmlGetProp(cur, "type");
			mask = xmlGetProp(cur, "allowmask");
			security_load_from_strings(&entry->security, type, mask);
			xmlFree(type);
			xmlFree(mask);
		}

		cur = cur->next;
	}
}

Entry*
entry_new_from_xml_node(xmlDocPtr doc, xmlNodePtr node) {
	Entry *entry;
	entry = entry_new();
	entry_load_from_xml_node(entry, doc, node);
	return entry;
}

static gboolean
entry_load_from_xml(Entry *entry, const char *data, int len, GError **err) {
	xmlNodePtr cur;
	xmlDocPtr  doc = NULL;
	xmlParserCtxtPtr ctxt;

	ctxt = xmlCreatePushParserCtxt(NULL, NULL,
				data, 4,
				NULL /* XXX why does this want a filename? */);
	/* suppress error messages */
	ctxt->sax->warning = NULL;
	ctxt->sax->error   = NULL;

	xmlParseChunk(ctxt, data+4, len-4, 0);
	xmlParseChunk(ctxt, data, 0, 1);
	if (!ctxt->errNo)
		doc = ctxt->myDoc;

	xmlFreeParserCtxt(ctxt);

	if (!doc) {
		/* XXX better error message. */
		g_set_error(err,
				0,
				0,
				_("Error parsing XML"));
		return FALSE;
	}

	cur = xmlDocGetRootElement(doc);
	entry_load_from_xml_node(entry, doc, cur);
	xmlFreeDoc(doc);
	return TRUE;
}

static gboolean
rfc822_get_keyval(const char **bufp, char *key, char *val) {
	const char *buf = *bufp;
	const char *p = buf;

	/* move p up to the colon after the key */
	while (*p != ':' && (p-buf < 100)) {
		if (*p == 0) return FALSE;
		if (*p == '\n') return FALSE;
		p++;
	}
	memcpy(key, buf, p-buf);
	key[p-buf] = 0;

	/* move p up to the head of the value */
	p++;
	while (*p == ' ') {
		if (*p == 0) return FALSE;
		if (*p == 0) return FALSE;
		p++;
	}
	/* scoot buf up to the newline, so the
	 * value lies between p and buf */
	buf = p;
	while (*buf != 0 && *buf != '\n' && (buf-p < 1000))
		buf++;
	memcpy(val, p, buf-p);
	val[buf-p] = 0;

	if (*buf != 0)
		*bufp = buf + 1;
	else
		*bufp = buf;
	return TRUE;
}

static gboolean
rfc822_load_entry(const char *key, const char *val, Entry *entry) {
#define RFC822_GET(A)                 \
    if (g_ascii_strcasecmp(key, #A) == 0) { \
        entry->A = g_strdup(val);   \
    }

	RFC822_GET(subject)
	else RFC822_GET(mood)
	else RFC822_GET(music)
	else RFC822_GET(pickeyword)
	else if (g_ascii_strcasecmp(key, "time") == 0) {
		ljdate_to_tm(val, &entry->time);
	}
	else return FALSE;

	return TRUE;
}

static gboolean
entry_load_from_rfc822(Entry *entry, const char *data, int len) {
	const char *buf = data;
	char key[1024], val[1024];

	/* if we don't get a keyval on the first line,
	 * we know we're not reading an rfc822 file. */
	if (!rfc822_get_keyval(&buf, key, val))
		return FALSE;
	/* if it's not a valid keyval,
	 * we know we're not reading an rfc822 file. */
	if (!rfc822_load_entry(key, val, entry))
		return FALSE;

	while (rfc822_get_keyval(&buf, key, val)) {
		rfc822_load_entry(key, val, entry);
	}

	/* buf is only advanced when we successfully get a keyval,
	 * so the first time we don't we're at the text of the entry. */
	while (*buf == '\n')
		buf++;
	entry->event = g_strdup(buf);

	return TRUE;
}

static gboolean
entry_load_from_plain(Entry *entry, const char *data, int len) {
	entry->event = g_strdup(data);
	return TRUE;
}

static gboolean
entry_load_from_autodetect(Entry *entry, EntryFileType *type, const char *data, int len, GError **err) {
	if (entry_load_from_xml(entry, data, len, err)) {
		*type = ENTRY_FILE_XML;
		return TRUE;
	}
	g_clear_error(err);
	if (entry_load_from_rfc822(entry, data, len)) {
		*type = ENTRY_FILE_RFC822;
		return TRUE;
	}
	*type = ENTRY_FILE_PLAIN;
	return entry_load_from_plain(entry, data, len);
}

gboolean
entry_load(Entry *entry, gchar *data, gsize len, EntryFileType *type, GError **err) {
	if (type == NULL) {
		EntryFileType type = ENTRY_FILE_AUTODETECT;
		return entry_load_from_autodetect(entry, &type, data, len, err);
	}

	switch (*type) {
		case ENTRY_FILE_XML:
			return entry_load_from_xml(entry, data, len, err);
		case ENTRY_FILE_RFC822:
			return entry_load_from_rfc822(entry, data, len);
		case ENTRY_FILE_PLAIN:
			return entry_load_from_plain(entry, data, len);
		case ENTRY_FILE_AUTODETECT:
		default:
			return entry_load_from_autodetect(entry, type, data, len, err);
	}
}

Entry*
entry_new_from_filename(const char *filename, EntryFileType *type, GError **err) {
	Entry *entry;
	gchar *data; gsize size;

	if (!g_file_get_contents(filename, &data, &size, err))
		return NULL;

	entry = entry_new();
	if (!entry_load(entry, data, size, type, err)) {
		g_free(data);
		entry_free(entry);
		return NULL;
	}

	g_free(data);
	return entry;
}

Entry*
entry_new_from_filename_xml(const char *filename, GError **err) {
	EntryFileType type = ENTRY_FILE_XML;
	return entry_new_from_filename(filename, &type, err);
}

gboolean
entry_to_xml_file(Entry *entry, const char *filename, GError **err) {
	int ret;
	xmlDocPtr doc;
	
	doc = entry_to_xml(entry);
	ret = xmlSaveFormatFileEnc(filename, doc, "utf-8", TRUE);
	xmlFreeDoc(doc);
	
	if (ret < 0) {
		/* XXX um, is there no way to get a more reasonable error message
		 * out of libxml? */
		g_set_error(err, 0, 0, _("Unable to save to file '%s'."), filename);
		return FALSE;
	}
	return TRUE;
}

/* the do/while kludge is to allow this macro to come in a blockless if */
#define _FILE_ERROR_THROW(err, message)                          \
	{                                                            \
		g_set_error(err, G_FILE_ERROR,                           \
				g_file_error_from_errno(errno), #message);       \
		return 0; /* works both as NULL and as FALSE, luckily */ \
	}

gboolean
entry_to_rfc822_file(Entry *entry, const char *filename, GError **err) {
	FILE *f = fopen(filename, "wb");
	if (!f)
		_FILE_ERROR_THROW(err, _("can't open rfc822 file"));

	/* XXX: add all other metadata fields */
	fprintf(f, "Subject: %s\n"
			"Mood: %s\n"
			"Music: %s\n"
			"PicKeyword: %s\n\n%s",
			entry->subject    ? entry->subject    : "", /* prevents */
			entry->mood       ? entry->mood       : "", /* "(null)" */
			entry->music      ? entry->music      : "",
			entry->pickeyword ? entry->pickeyword : "",
			entry->event      ? entry->event      : "");

	if (entry->time.tm_year) {
		char *ljdate;
		ljdate = tm_to_ljdate(&entry->time);
		fprintf(f, "Time: %s\n", ljdate);
		g_free(ljdate);
	}
	
	if (fclose(f) != 0)
		_FILE_ERROR_THROW(err, _("can't close rfc822 file"));
	
	return TRUE;
}

/* give the user a chance to edit an Entry with their favorite editor.
 * loads either an empty entry, one partially specified on the command
 * line, or whatever is in the draft, whichever is appropriate. Should
 * work correctly with -c (postcli) cmdline option.
 *
 * XXX update docs here
 * Note how unlike the entry_from_* functions, this acts on an out
 * variable containing an *existing* Entry. see entry_from_user_editor
 * for the lower-level service of forking off an editor on a file and
 * constructing an Entry as a result. */
gboolean
entry_edit_with_usereditor(Entry *entry, GError **err) {
	gchar   editfn[1024];
	gchar   draftpath[1024];
	Entry  *tmp_entry = NULL;
	gint    editfh;
	gchar   clean_headers[] =
		"Subject: \n"
		"Mood: \n"
		"Music: \n"
		"PicKeyword: \n\n";
	
	conf_make_path("draft", draftpath, 1024);

	conf_make_path("edit-XXXXXX", editfn, 1024);
	editfh = g_mkstemp(editfn);
	if (!editfh)
		_FILE_ERROR_THROW(err, _("can't make temp file"));
	
	if (conf.options.autosave && app.autoloaddraft_ok &&
			((tmp_entry = entry_new_from_filename_xml(draftpath, NULL)) != NULL)) {
		/* we're about to open the same file again */
		if (close(editfh) != 0)
			_FILE_ERROR_THROW(err, _("can't close temp file"));
		if (!entry_to_rfc822_file(tmp_entry, editfn, err))
			return FALSE; /* err has already been set */
		entry_free(tmp_entry); /* and _not_ g_free */
		
	} else if (entry->itemid != 0) { /* partially constructed (-s etc.) */
		/* we're about to open the same file again */
		if (close(editfh) != 0)
			_FILE_ERROR_THROW(err, _("can't close temp file"));
		if (!entry_to_rfc822_file(entry, editfn, err))
			return FALSE; /* err has already been set */
		
	} else {                         /* new entry */
		if (!write(editfh, clean_headers, strlen(clean_headers)))
			_FILE_ERROR_THROW(err, _("can't write to temp file"));
		if (close(editfh) != 0)
			_FILE_ERROR_THROW(err, _("can't close temp file"));
	}

	tmp_entry = entry_from_user_editor(editfn, err);
	memcpy(entry, tmp_entry, sizeof(Entry)); /* the ol' switcheroo */
	g_free(tmp_entry);                       /* and _not_ entry_free */

	if (unlink(editfn) != 0)
		_FILE_ERROR_THROW(err, _("can't unlink temp file"));

	return TRUE;
}

static Entry*
entry_from_user_editor(const char *filename, GError **err) {
#ifndef G_OS_WIN32
	gint   pid;
	Entry *entry;
	EntryFileType type;
	
	/* g_spawn* would do no good: it disassociates the tty. viva fork! */
	pid = fork();
	if (pid < 0) {                 /* fork error */
		g_set_error(err, G_SPAWN_ERROR, G_SPAWN_ERROR_FORK,
				g_strerror(errno));
		return NULL;
	}

	if (pid == 0) {                /* child */
		gchar *editor =
			(getenv("VISUAL") ? getenv("VISUAL") :
			 getenv("EDITOR") ? getenv("EDITOR") : "vi");
		execlp(editor, editor, filename, NULL);
		_exit(0);
	}

	/* parent */
	if (wait(NULL) != pid) {
		g_set_error(err, G_SPAWN_ERROR, G_SPAWN_ERROR_FAILED,
				g_strerror(errno));
		return NULL;
	}

	type = ENTRY_FILE_RFC822;
	if (!(entry = entry_new_from_filename(filename, &type, err))) {
		return NULL; /* err has already been already set; propagate it */
	}
	
	app.autoloaddraft_ok = FALSE;

	return entry;
	
#else
	g_warning("external editor option not supported on Win32 yet. Sorry.\n");
	return NULL;
#endif /* ndef G_OS_WIN32 */
}
