/* logjam - a GTK client for LiveJournal.
 * Copyright (C) 2000-2003 Evan Martin <evan@livejournal.com>
 *
 * vim: tabstop=4 shiftwidth=4 noexpandtab :
 */

#include "config.h"
#include <glib.h>

#include <stdio.h>
#include <errno.h>
#include <libxml/tree.h>
#include <stdlib.h>  /* atoi. */
#include <string.h>  /* strchr. */
#include "util.h"
#include "network.h"
#include "security.h"
#include "livejournal.h"
#include "lj_xml.h"

char*
nid_by_id(GList *l, int id) {
	for ( ; l; l = l->next) {
		if (((NameIDHash*)l->data)->id == id) 
			return ((NameIDHash*)l->data)->name;
	}
	return NULL;
}
int
nid_by_name(GList *l, const char* name) {
	for ( ; l; l = l->next) {
		if (strcmp(((NameIDHash*)l->data)->name, name) == 0) 
			return ((NameIDHash*)l->data)->id;
	}
	return -1;
}
gint
nid_compare_alpha(NameIDHash *a, NameIDHash *b) {
	return g_ascii_strcasecmp(a->name, b->name);
}

void
nid_free(NameIDHash *nid) {
	g_free(nid->name);
	g_free(nid);
}

gint
user_compare(gconstpointer a, gconstpointer b) {
	return g_ascii_strcasecmp(((User*)a)->username, ((User*)b)->username);
}

Server*
server_new(void) {
	Server *s = g_new0(Server, 1);
	/* default to protocol version 1. */
	s->protocolversion = 1;
	return s;
}

User*
server_get_user_by_username(Server *server, const char *username) {
	GList *l;
	for (l = server->users; l != NULL; l = l->next) {
		if (strcmp(username, ((User*)l->data)->username) == 0) {
			return l->data;
		}
	}
	return NULL;
}

void
server_add_user(Server *server, User *user) {
	server->users = g_list_insert_sorted(server->users, user, user_compare);
}

int 
server_get_last_cached_moodid(Server *server) {
	GList *l;
	int max = 0;
	for (l = server->moods; l != NULL; l = l->next) {
		if (((Mood*)l->data)->id > max) 
			max = ((Mood*)l->data)->id;
	}
	return max;
}

Friend*
friend_new(void) {
	Friend *f = g_new0(Friend, 1);
	strcpy(f->foreground, "#000000");
	strcpy(f->background, "#FFFFFF");
	return f;
}

FriendType
friend_type_from_str(char *str) {
	if (str) {
		if (strcmp(str, "community") == 0) 
			return FRIEND_TYPE_COMMUNITY;
	} 
	return FRIEND_TYPE_USER;
}

void
friend_free(Friend *f) {
	g_free(f->username);
	g_free(f->fullname);
	g_free(f);
}

gint
friend_compare_username(gconstpointer a, gconstpointer b) {
	const Friend *fa = a;
	const Friend *fb = b;

	return strcmp(fa->username, fb->username);
}

/*
boxed friend type?
   
static Friend*
friend_copy(Friend *o) {
	Friend *f = g_new0(Friend, 1);
	f->username = g_strdup(o->username);
	f->fullname = g_strdup(o->fullname);
	f->foreground = o->foreground;
	f->background = o->background;
	f->conn = o->conn;
	f->type = o->type;
	f->groupmask = o->groupmask;
	return f;
}

GType friend_get_gtype(void) {
	static GType type = 0;
	if (type == 0)
		type = g_boxed_type_register_static("Friend",
				(GBoxedCopyFunc) friend_copy,
				(GBoxedFreeFunc) friend_free);
	return type;
}*/

void 
security_append_to_request(Security *security, NetRequest *request) {
	char *text = NULL;

	switch (security->type) {
		case SECURITY_PUBLIC:
			text = "public"; break;
		case SECURITY_PRIVATE:
			text = "private"; break;
		case SECURITY_FRIENDS:
		case SECURITY_CUSTOM:
			text = "usemask"; break;
	}
	net_request_copys(request, "security", text);

	if (security->type == SECURITY_FRIENDS) {
		net_request_seti(request, "allowmask", SECURITY_ALLOWMASK_FRIENDS);
	} else if (security->type == SECURITY_CUSTOM) {
		net_request_seti(request, "allowmask", security->allowmask);
	}
}

void
security_load_from_strings(Security *security, const char *sectext, const char *allowmask) {
	if (!sectext) {
		security->type = SECURITY_PUBLIC;
		return;
	}

	if (g_ascii_strcasecmp(sectext, "public") == 0) {
		security->type = SECURITY_PUBLIC;
	} else if (g_ascii_strcasecmp(sectext, "private") == 0) {
		security->type = SECURITY_PRIVATE;
	} else if (g_ascii_strcasecmp(sectext, "friends") == 0) {
		security->type = SECURITY_FRIENDS;
	} else if (g_ascii_strcasecmp(sectext, "usemask") == 0 ||
			   g_ascii_strcasecmp(sectext, "custom") == 0) {
		unsigned int am = 0;
		if (allowmask)
			am = atoi(allowmask);

		switch (am) {
			case 0:
				security->type = SECURITY_PRIVATE; break;
			case SECURITY_ALLOWMASK_FRIENDS: 
				security->type = SECURITY_FRIENDS; break;
			default:
				security->type = SECURITY_CUSTOM;
				security->allowmask = am;
		}
	} else {
		g_warning("security: '%s' unhandled", sectext);
	}
}

void 
security_load_from_result(Security *security, NetResult *result) {
	char *sectext, *allowmask;

	sectext = net_result_get(result, "security");
	if (sectext) {
		allowmask = net_result_get(result, "allowmask");
		security_load_from_strings(security, sectext, allowmask);
	}
}

void
security_save_to_strings(Security *security, char **sectext, char **allowmask) {
	char *type = NULL;
	switch (security->type) {
		case SECURITY_PUBLIC: return;
		case SECURITY_PRIVATE: type = "private"; break;
		case SECURITY_FRIENDS: type = "friends"; break;
		case SECURITY_CUSTOM: type = "custom"; break;
	}
	if (sectext)
		*sectext = g_strdup(type);

	if (allowmask && security->type == SECURITY_CUSTOM)
		*allowmask = g_strdup_printf("%ud", security->allowmask);
}

