/* logjam - a GTK client for LiveJournal.
 * Copyright (C) 2000-2003 Evan Martin <evan@livejournal.com>
 *
 * vim: tabstop=4 shiftwidth=4 noexpandtab :
 */

#ifndef __LIVEJOURNAL_H__
#define __LIVEJOURNAL_H__

/* structures for representing livejournal data. 
 * logjam-specific structures are in conf.h.
 */ 
#include <time.h> /* post timestamp */
#include <glib-object.h>
#include <libxml/tree.h> /* entry to xml node. */
#include "network.h" /* request */

/* associate numbers<->names in a two-way structure (using a GList). */
#define NID_CONTENT int id; char *name; /* yay for no inheritance in C. */
typedef struct {
	NID_CONTENT
} NameIDHash;
char* nid_by_id(GList *l, int id);
int   nid_by_name(GList *l, const char* name);
int   nid_compare_alpha(NameIDHash *a, NameIDHash *b);
void  nid_free(NameIDHash *nid);
#define mood_id_from_name(name) nid_by_name(conf_cur_server()->moods, name)
#define mood_name_from_id(id) nid_by_id(conf_cur_server()->moods, id)
typedef NameIDHash Mood;

/* --- friend group --- */

typedef struct {
	NID_CONTENT
	gboolean ispublic;
} FriendGroup;
#define friendgroup_from_id(id) nid_by_id(conf_cur_user()->friendgroups, id)
#define friend_group_free nid_free

/* --- user --- */

typedef struct {
	char *username;
	char *fullname;
	char *password;
	gboolean remember_user, remember_password;
	guint32 cfmask;

	GList *pickws;
	GList *friendgroups;
	GList *usejournals;
} User;

gint user_compare(gconstpointer a, gconstpointer b);

/* --- server --- */

typedef struct {
	char *name;
	char *url;

	GList *users;
	User *usercur;

	GList *moods;

	int protocolversion;
} Server;

Server* server_new(void);
User* server_get_user_by_username(Server *server, const char *username);
void  server_add_user(Server *server, User *user);
int   server_get_last_cached_moodid(Server *server);

/* --- security --- */

typedef enum {
	SECURITY_PUBLIC = 0,
	SECURITY_FRIENDS,
	SECURITY_PRIVATE,
	SECURITY_CUSTOM
} SecurityType;

typedef struct {
	SecurityType type;
	unsigned int allowmask;
} Security;
/* an allowmask of 1 means friends-only. */
#define SECURITY_ALLOWMASK_FRIENDS 1

void security_append_to_request(Security *security, GHashTable *request);
void security_load_from_request(Security *security, GHashTable *request);
void security_load_from_strings(Security *security, const char *sectext, const char *allowmask);
void security_save_to_strings(Security *security, char **sectext, char **allowmask);

/* --- friend --- */

#define FRIEND_CONN_MY   (1 << 0)
#define FRIEND_CONN_OF   (1 << 1)
#define FRIEND_CONN_BOTH (FRIEND_CONN_MY | FRIEND_CONN_OF)

typedef enum {
	FRIEND_TYPE_USER=1,
	FRIEND_TYPE_COMMUNITY
} FriendType;

typedef struct {
	char *username;
	char *fullname;
	char foreground[8], background[8];
	int conn; /* FRIEND_MY, etc */
	FriendType type; 
	guint32 groupmask;
} Friend;

/* the first bit is the "all friends" group. */
#define FRIEND_GROUP_ALLFRIENDS 1
Friend *friend_new(void);
Friend *friend_new_with(char *username, char *name, char *fg, char *bg, int conn, guint mask, char *type);
void    friend_free(Friend *f);
FriendType friend_type_from_str(char *str);
gint friend_compare_username(gconstpointer a, gconstpointer b);

/* we make friends into a boxed type so we can use them in our list view.
 * don't confuse the "type" here (a glib object type)
 * with the "type" above (the user/community distinction)! */
GType friend_get_gtype(void);
#define FRIEND_GTYPE friend_get_gtype()

/* --- comments --- */

typedef enum {
	COMMENTS_DEFAULT,
	COMMENTS_NOEMAIL,
	COMMENTS_DISABLE
} CommentsType;

/* -- entry --- */

typedef struct {
	/*        < 0: draft
	 * itemid = 0: unknown itemid
	 *        > 0: known itemid
	 */
	int itemid;
	char *subject;
	char *event;

	char *mood, *music, *pickeyword;
	int preformatted;
	int backdated;
	CommentsType comments;

	struct tm time;
	Security security;
} Entry;

typedef enum {
	ENTRY_FILE_AUTODETECT,
	ENTRY_FILE_XML,
	ENTRY_FILE_RFC822,
	ENTRY_FILE_PLAIN
} EntryFileType;

Entry*   entry_new                  (void);
Entry*   entry_new_from_filename_xml(const char *filename, GError **err);
Entry*   entry_new_from_filename    (const char *filename, EntryFileType *type,
                                     GError **err);
Entry*   entry_new_from_stdin       (GError **err);
Entry*   entry_new_from_xml_node    (xmlDocPtr doc, xmlNodePtr node);

Entry*   entry_new_from_result      (NetResult *result, int i);
Entry*   entry_new_single_from_result(NetResult *result);

Entry*   entry_copy                 (Entry *e);
void     entry_free                 (Entry *e);

gboolean entry_load                 (Entry *entry, gchar *data, gsize len,
                                     EntryFileType *type, GError **err);
char*    entry_make_summary         (Entry *entry);
void     entry_set_request_fields   (Entry *entry, NetRequest *request);

xmlNodePtr entry_to_xml_node        (Entry *entry, xmlDocPtr doc);
gboolean entry_to_xml_file          (Entry *entry, const char *filename,
                                     GError **err);
gboolean entry_to_rfc822_file       (Entry *entry, const char *filename,
                                     GError **err);

gboolean entry_edit_with_usereditor (Entry *entry, GError **err);

void     entry_load_metadata        (Entry *entry, const char *key, const char *value);

GHashTable* entries_new_from_result(NetResult *result, int count, Entry **plastentry);

#endif /* __LIVEJOURNAL_H__ */
