/* logjam - a GTK client for LiveJournal.
 * Copyright (C) 2000-2003 Evan Martin <evan@livejournal.com>
 *
 * vim: tabstop=4 shiftwidth=4 noexpandtab :
 */

#include "config.h"

#include <gtk/gtk.h>
#include <glib.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "conf.h"
#include "network.h"
#include "menu.h"
#include "lj.h"
#include "friends.h"
#include "history.h"
#include "util.h"
#include "spawn.h"
#include "security.h"
#include "icons.h"
#include "meta.h"
#include "settings.h"
#include "pollcreator.h"
#include "tools.h"
#include "undo-manager.h"
#ifdef HAVE_GTKHTML
#include "preview.h"
#endif
#include "sync.h"

enum {
	ACTION_NONE=0,
	ACTION_JOURNAL,
	ACTION_WEB_LINKS,

	ACTION_UNDO,
	ACTION_REDO,
	
	ACTION_CUT,
	ACTION_COPY,
	ACTION_PASTE,
	
	ACTION_ENTRY_SUBMIT_SEP,
	ACTION_ENTRY_SUBMIT_NEW,
	ACTION_ENTRY_SAVE_SERVER,

	ACTION_ENTRY_SECURITY_PUBLIC, 
	ACTION_ENTRY_SECURITY_FRIENDS,
	ACTION_ENTRY_SECURITY_PRIVATE,
	ACTION_ENTRY_SECURITY_CUSTOM, 

	ACTION_ENTRY_COMMENTS_DEFAULT,
	ACTION_ENTRY_COMMENTS_NOEMAIL,
	ACTION_ENTRY_COMMENTS_DISABLE,

	ACTION_ENTRY_BACKDATED,
	ACTION_ENTRY_PREFORMATTED,

	ACTION_ENTRY_PICTURE,

	ACTION_JOURNAL_POSTAS,
	ACTION_JOURNAL_POSTAS_SEP,
	ACTION_LOGIN,
	ACTION_MANAGER,

	ACTION_TOOLS_CONSOLE
};

#define KEY_PICTUREKEYWORD "picturekeyword"

void about_dlg(GtkWidget *mainwin);
void console_dialog_run(GtkWidget *parent);

static void
menu_make_link(LJWin *ljw) {
	void link_dialog_run(GtkWindow *win, LogJamDoc *doc);
	link_dialog_run(GTK_WINDOW(ljw), ljw->doc);
}
static void
menu_make_journal_link(LJWin *ljw) {
	void link_lj_dialog_run(GtkWindow *win, LogJamDoc *doc);
	link_lj_dialog_run(GTK_WINDOW(ljw), ljw->doc);
}

static void
new_cb(LJWin *ljw) {
	if (!lj_confirm_lose_entry(ljw)) return;
	lj_clear_entry(ljw);
}
static void
load_last_entry(LJWin *ljw) { 
	Entry *entry;

	if (!lj_confirm_lose_entry(ljw)) return;

	entry = history_load_latest(GTK_WINDOW(ljw));
	if (!entry)
		return;

	logjam_doc_load_entry(ljw->doc, entry);
	entry_free(entry);
}
static void
run_history_recent_dlg(LJWin *ljw) { 
	Entry *entry;

	if (!lj_confirm_lose_entry(ljw)) return;

	if ((entry = history_recent_dialog_run(GTK_WINDOW(ljw))) == NULL)
		return;

	logjam_doc_load_entry(ljw->doc, entry);
	entry_free(entry);
}

static void
run_offline_dlg(LJWin *ljw) { 
	Entry *offline_dlg_run(GtkWindow *);
	Entry *entry;

	if (!lj_confirm_lose_entry(ljw)) return;

	if ((entry = offline_dlg_run(GTK_WINDOW(ljw))) == NULL)
		return;

	logjam_doc_load_entry(ljw->doc, entry);
	entry_free(entry);
}

static void
loginout_cb(LJWin *ljw) {
	lj_do_loginout(ljw);
}

static void menu_save(LJWin *ljw) { lj_save(ljw); }
static void menu_save_as_file(LJWin *ljw) { lj_save_as_file(ljw); }
static void menu_save_as_draft(LJWin *ljw) { lj_save_as_draft(ljw); }

static void
security_cb(LJWin *ljw, int action, GtkCheckMenuItem *item) {
	Security sec = logjam_doc_get_security(ljw->doc);

	if (!gtk_check_menu_item_get_active(item))
		return;

	sec.type = action - ACTION_ENTRY_SECURITY_PUBLIC;
	if (sec.type == SECURITY_CUSTOM) {
		sec.allowmask = custom_security_dlg_run(GTK_WINDOW(ljw),
					sec.allowmask, NULL);
		/* if they didn't include anyone, it's a private entry. */
		if (sec.allowmask == 0)
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(
						gtk_item_factory_get_widget_by_action(ljw->factory,
							ACTION_ENTRY_SECURITY_PRIVATE)), TRUE);
	}

	logjam_doc_set_security(ljw->doc, &sec);
}

static void
comments_cb(LJWin *ljw, int action, GtkCheckMenuItem *item) {
	if (!gtk_check_menu_item_get_active(item))
		return;
	logjam_doc_set_comments(ljw->doc, action - ACTION_ENTRY_COMMENTS_DEFAULT);
}

static void
backdated_cb(LJWin *ljw, int action, GtkCheckMenuItem *item) {
	logjam_doc_set_backdated(ljw->doc, gtk_check_menu_item_get_active(item));
}

static void
preformatted_cb(LJWin *ljw, int action, GtkCheckMenuItem *item) {
	logjam_doc_set_preformatted(ljw->doc, gtk_check_menu_item_get_active(item));
}
static void
picture_cb(GObject *item, LJWin *ljw) {
	char *pic;
	if (!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(item)))
		return;
	pic = g_object_get_data(item, KEY_PICTUREKEYWORD);
	logjam_doc_set_pickeyword(ljw->doc, pic);
}

static GtkWidget*
menu_item_from_action(GtkItemFactory *item_factory, int action) {
	GtkWidget *menu, *item;
	menu = gtk_item_factory_get_widget_by_action(item_factory, action);
	item = gtk_menu_get_attach_widget(GTK_MENU(menu));
	return item;
}

void
radio_join_group(GtkItemFactory *factory, int i, ...) {
	GtkWidget *firstradio, *radio;
	GSList *group = NULL;
	va_list ap;

	firstradio = radio = gtk_item_factory_get_widget_by_action(factory, i);
	group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(radio));

	va_start(ap, i);
	i = va_arg(ap, int);
	while (i > 0) {
		radio = gtk_item_factory_get_widget_by_action(factory, i);
		gtk_radio_menu_item_set_group(GTK_RADIO_MENU_ITEM(radio), group);
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(radio), FALSE);
		group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(radio));
		i = va_arg(ap, int);
	}
	va_end(ap);

	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(firstradio), TRUE);
}

static void
cutcopypaste_cb(LJWin *ljw, int action) {
	GtkWidget *target;
	const char* signalname;
	guint signalid;

	switch (action) {
		case ACTION_CUT:   signalname = "cut_clipboard";   break;
		case ACTION_COPY:  signalname = "copy_clipboard";  break;
		case ACTION_PASTE: signalname = "paste_clipboard"; break;
		default: return;
	}

	/* verify that the focused widget actually responds to this signal
	 * before emitting it, because if it doesn't we get an ugly error. */
	target = gtk_window_get_focus(GTK_WINDOW(ljw));
	signalid = g_signal_lookup(signalname, G_OBJECT_TYPE(target));
	if (signalid)
		g_signal_emit(target, signalid, 0); /* the last param appears to only
	                                           be used for property signals. */ 
}

static void
undoredo_cb(LJWin *ljw, int action) {
	if (action == ACTION_UNDO) {
		if (undo_manager_can_undo(ljw->um)) {
			undo_manager_undo(ljw->um);
		}
	} else if (action == ACTION_REDO) {
		if (undo_manager_can_redo(ljw->um)) {
			undo_manager_redo(ljw->um);
		}
	}
}
	
static void
menu_insert_file(LJWin *ljw) {
	tools_insert_file(GTK_WINDOW(ljw), ljw->doc);
}
static void
menu_html_escape(LJWin *ljw) {
	tools_html_escape(GTK_WINDOW(ljw), ljw->doc);
}
static void
menu_remove_linebreaks(LJWin *ljw) {
	tools_remove_linebreaks(GTK_WINDOW(ljw), ljw->doc);
}
static void
menu_ljcut(LJWin *ljw) {
	tools_ljcut(GTK_WINDOW(ljw), ljw->doc);
}
static void
menu_validate_xml(LJWin *ljw) {
	tools_validate_xml(GTK_WINDOW(ljw), ljw->doc);
}

void manager_dialog(GtkWidget *parent);

GtkWidget* 
menu_make_bar(LJWin *ljw) {
	GtkWidget *bar, *menu;
	GtkAccelGroup *accelgroup = NULL;
	GtkItemFactory *item_factory = NULL;

/* a note on accelerators:
 *  shift-ctl-[x]  is used to type in UTF-8 chars; 
 *  this means we can't use any shift-ctl accels. */
static GtkItemFactoryEntry menu_items[] = {
{ N_("/_LogJam"),                      NULL, NULL, 0, "<Branch>" },
{ N_("/LogJam/_About LogJam..."),      NULL, about_dlg, 0, "<StockItem>", "logjam-goat" },
{ N_("/LogJam/_User Manager..."),      NULL, manager_dialog, ACTION_MANAGER, NULL },
{ N_("/LogJam/_Login..."),             NULL, loginout_cb, ACTION_LOGIN, NULL },
{ N_("/LogJam/_Preferences..."),       NULL, settings_run, SETTINGS_PAGE_UNSPECIFIED, "<StockItem>", GTK_STOCK_PREFERENCES },
{    "/LogJam/---",                    NULL, NULL, 0, "<Separator>" },
{ N_("/LogJam/_Quit"),                 "<ctl>Q", lj_quit, 0, "<StockItem>", GTK_STOCK_QUIT },

{ N_("/E_ntry"),                       NULL, NULL, 0, "<Branch>" },
{ N_("/Entry/_New"),                   NULL, new_cb, 0, "<StockItem>", GTK_STOCK_NEW },
{ N_("/Entry/_Open"),                  NULL, NULL, 0, "<Branch>" },
{ N_("/Entry/Open/File..."),           NULL, lj_open_entry, 0, "<StockItem>", GTK_STOCK_OPEN },
{ N_("/Entry/Open/Draft..."),          NULL, lj_open_draft, 0, NULL },
{ N_("/Entry/_Save"),                  NULL, menu_save, 0, "<StockItem>", GTK_STOCK_SAVE },
{ N_("/Entry/Save _As"),               NULL, NULL, 0, "<Branch>" },
{ N_("/Entry/Save As/_File..."),       NULL, menu_save_as_file, 0, "<StockItem>", GTK_STOCK_SAVE_AS },
{ N_("/Entry/Save As/_Draft..."),      NULL, menu_save_as_draft, 0, NULL },
{    "/Entry/---",                     NULL, NULL, 0, "<Separator>" },

{ N_("/Entry/_Security"),              NULL, NULL, 0, "<Branch>" },
{ N_("/Entry/Security/_Public"),       NULL,
	security_cb, ACTION_ENTRY_SECURITY_PUBLIC,  "<RadioItem>" },
{ N_("/Entry/Security/_Friends-Only"), NULL,
	security_cb, ACTION_ENTRY_SECURITY_FRIENDS, "<RadioItem>" },
{ N_("/Entry/Security/P_rivate"),      NULL,
	security_cb, ACTION_ENTRY_SECURITY_PRIVATE, "<RadioItem>" },
{ N_("/Entry/Security/_Custom..."),    NULL,
	security_cb, ACTION_ENTRY_SECURITY_CUSTOM,  "<RadioItem>" },
{ N_("/Entry/_Picture"), NULL, NULL, ACTION_ENTRY_PICTURE, "<Branch>" },
{ N_("/Entry/_Comments"), NULL, NULL, 0, "<Branch>" },
{ N_("/Entry/Comments/_Default"),      NULL,
	comments_cb, ACTION_ENTRY_COMMENTS_DEFAULT, "<RadioItem>" },
{ N_("/Entry/Comments/Don't _Email"),  NULL,
	comments_cb, ACTION_ENTRY_COMMENTS_NOEMAIL, "<RadioItem>" },
{ N_("/Entry/Comments/D_isable"),      NULL,
	comments_cb, ACTION_ENTRY_COMMENTS_DISABLE, "<RadioItem>" },
{ N_("/Entry/_Options"),               NULL, NULL, 0, "<Branch>" },
{ N_("/Entry/Options/_Backdated"),     NULL,
	backdated_cb, ACTION_ENTRY_BACKDATED, "<CheckItem>" },
{ N_("/Entry/Options/_Preformatted"),  NULL,
	preformatted_cb, ACTION_ENTRY_PREFORMATTED, "<CheckItem>" },
{    "/Entry/---",                     NULL, NULL, 0, "<Separator>" },
{ N_("/Entry/_Validate XML"),    NULL, menu_validate_xml, 0, NULL },
#ifdef HAVE_GTKHTML
{ N_("/Entry/_Preview HTML..."), NULL, preview_ui_show, 0, NULL },
#endif
{    "/Entry/---",                     NULL, NULL, ACTION_ENTRY_SUBMIT_SEP, "<Separator>" },
{ N_("/Entry/S_ubmit as New"),         NULL, lj_submit_entry, ACTION_ENTRY_SUBMIT_NEW, NULL },
{ N_("/Entry/_Save Changes to Server"), NULL, lj_save_entry_server, ACTION_ENTRY_SAVE_SERVER, NULL },

{ N_("/_Edit"),       NULL, NULL, 0, "<Branch>" },
{ N_("/Edit/_Undo"),  "<ctl>Z", undoredo_cb,
	ACTION_UNDO,	"<StockItem>", GTK_STOCK_UNDO },
{ N_("/Edit/_Redo"),  "<ctl>R", undoredo_cb,
	ACTION_REDO,	"<StockItem>", GTK_STOCK_REDO },
{    "/Edit/---",     NULL, NULL, 0, "<Separator>" },
{ N_("/Edit/Cu_t"),   NULL, cutcopypaste_cb,
	ACTION_CUT,   "<StockItem>", GTK_STOCK_CUT },
{ N_("/Edit/_Copy"),  NULL, cutcopypaste_cb, 
	ACTION_COPY,  "<StockItem>", GTK_STOCK_COPY },
{ N_("/Edit/_Paste"), NULL, cutcopypaste_cb,
	ACTION_PASTE, "<StockItem>", GTK_STOCK_PASTE },
{    "/Edit/---",     NULL, NULL, 0, "<Separator>" },
{ N_("/Edit/_HTML Escape"),       NULL, menu_html_escape, 0, NULL },
{ N_("/Edit/_Remove Linebreaks"), NULL, menu_remove_linebreaks, 0, NULL },

{ N_("/_Insert"),                 NULL, NULL, 0, "<Branch>" },
{ N_("/Insert/_Poll..."),  NULL, run_poll_creator_dlg, 0, NULL },
{ N_("/Insert/_File..."),  NULL, menu_insert_file, 0, NULL },
{    "/Insert/---",               NULL, NULL, 0, "<Separator>" },
{ N_("/Insert/_Link..."),         "<ctl>L",       menu_make_link },
{ N_("/Insert/_Journal Link..."), "<ctl><alt>L", menu_make_journal_link },
{ N_("/Insert/lj-_cut..."),       "<ctl><alt>X", menu_ljcut, 0, NULL },

{ N_("/_Journal"),                      NULL,          NULL, ACTION_JOURNAL, "<Branch>" },
{ N_("/Journal/_Friends..."),           "<ctl><alt>F", friends_manager_show, 0, NULL },
{    "/Journal/---",                    NULL, NULL, 0, "<Separator>" },
{ N_("/Journal/_Load Last Entry"),      NULL,          load_last_entry, 0, NULL },
{ N_("/Journal/_Load Recent Entry..."), NULL,          run_history_recent_dlg, 0, NULL },
{ N_("/Journal/_Load Offline Entry..."), NULL,         run_offline_dlg, 0, NULL },
{ N_("/Journal/_Synchronize Offline Copy..."), NULL, sync_run, 0, NULL },
{    "/Journal/---",                      NULL, NULL, 0, "<Separator>" },
{ N_("/Journal/_LiveJournal Console..."), NULL, console_dialog_run, ACTION_TOOLS_CONSOLE, NULL },
{ N_("/Journal/_Web Links"),              NULL, NULL, ACTION_WEB_LINKS, "<Branch>" },

};
	int itemcount = sizeof(menu_items) / sizeof(menu_items[0]);

	accelgroup = gtk_accel_group_new();
	ljw->factory = item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR,
			"<main>", accelgroup);
	gtk_item_factory_set_translate_func(ljw->factory, gettext_translate_func, NULL, NULL);
	gtk_item_factory_create_items(item_factory, itemcount, menu_items, ljw);
	gtk_window_add_accel_group(GTK_WINDOW(ljw), accelgroup);

	bar = gtk_item_factory_get_widget(item_factory, "<main>");

	menu = menu_item_from_action(item_factory, ACTION_JOURNAL);
	gtk_widget_set_sensitive(menu, FALSE);

	ljw->mweb = gtk_item_factory_get_item_by_action(ljw->factory, ACTION_WEB_LINKS);
	//ljw->mweb = menu_item_from_action(item_factory, ACTION_WEB_LINKS);
	//gtk_widget_hide(ljw->mweb);
	
	ljw->msubmitsep = gtk_item_factory_get_widget_by_action(item_factory,
			ACTION_ENTRY_SUBMIT_SEP);
	ljw->msubmit = gtk_item_factory_get_widget_by_action(item_factory,
			ACTION_ENTRY_SUBMIT_NEW);
	ljw->msaveserver = gtk_item_factory_get_widget_by_action(item_factory,
			ACTION_ENTRY_SAVE_SERVER);
	gtk_widget_hide(ljw->msaveserver);

	ljw->mloginout = gtk_item_factory_get_widget_by_action(item_factory,
			ACTION_LOGIN);

	radio_join_group(item_factory, ACTION_ENTRY_SECURITY_PUBLIC, 
	                               ACTION_ENTRY_SECURITY_FRIENDS,
	                               ACTION_ENTRY_SECURITY_PRIVATE,
	                               ACTION_ENTRY_SECURITY_CUSTOM,
	                               -1);

	radio_join_group(item_factory, ACTION_ENTRY_COMMENTS_DEFAULT,
	                               ACTION_ENTRY_COMMENTS_NOEMAIL,
	                               ACTION_ENTRY_COMMENTS_DISABLE,
	                               -1);

	ljw->mpicture = menu_item_from_action(item_factory,
			ACTION_ENTRY_PICTURE);

	ljw->mundo = gtk_item_factory_get_widget_by_action(item_factory,
			ACTION_UNDO);
	gtk_widget_set_sensitive(ljw->mundo, undo_manager_can_undo(ljw->um));
	ljw->mredo = gtk_item_factory_get_widget_by_action(item_factory,
			ACTION_REDO);
	gtk_widget_set_sensitive(ljw->mredo, undo_manager_can_redo(ljw->um));

	return bar;
}

static void
menu_spawn_url(GtkWidget *w, GtkWindow *ljw) {
	char *url;
	url = g_object_get_data(G_OBJECT(w), "url");
	if (url)
		spawn_url(ljw, url);
}

static GtkWidget*
web_recurse(LJWin *ljw, NetResult *hash, int base) {
	GtkWidget *menu, *item;
	char *text, *str, *url, *sub, *dupstr;
	int i, count;

	str = net_result_getf(hash, "menu_%d_count", base);
	if (str == NULL) {
		g_warning("menu with no elements?");
		return NULL;
	}
	count = atoi(str);

	menu = gtk_menu_new();

	for (i = 1; i < count+1; i++) {
		text = net_result_getf(hash, "menu_%d_%d_text", base, i);
		if (text == NULL) {
			g_warning("menu item has no text?");
			continue;
		}

		url = net_result_getf(hash, "menu_%d_%d_url", base, i);
		sub = net_result_getf(hash, "menu_%d_%d_sub", base, i);

		if (text[0] == '-' && text[1] == 0) {
			/* separator */
			item = gtk_menu_item_new();
			gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
		} else if (url != NULL) {
			/* url menu item */
			item = gtk_image_menu_item_new_with_label(text);
			gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
					gtk_image_new_from_stock(GTK_STOCK_JUMP_TO, GTK_ICON_SIZE_MENU));

			dupstr = g_strdup(url);
			/* free the dup'd string after the menu item is destroyed! */
			g_object_set_data_full(G_OBJECT(item), "url",
					dupstr, g_free);
			g_signal_connect(G_OBJECT(item), "activate",
					G_CALLBACK(menu_spawn_url), ljw);
			gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
		} else if (sub != NULL) {
			/* submenu ! */
			GtkWidget *submenu;
			int subbase;

			subbase = atoi(sub);
			item = gtk_menu_item_new_with_mnemonic(text);
			submenu = web_recurse(ljw, hash, subbase);
			gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
			gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
		} else {
			g_warning("unknown menu item type...");
		}
	}
	return menu;
}

static void 
menu_update_web(LJWin *ljw, NetResult *hash) {
	if (hash) {
		gtk_menu_item_set_submenu(GTK_MENU_ITEM(ljw->mweb),
				web_recurse(ljw, hash, 0));
		gtk_widget_show_all(ljw->mweb);
	} else {
		gtk_widget_hide(ljw->mweb);
	}
}

static GtkWidget*
make_picture_menu(LJWin *ljw, Entry *entry) {
	GtkWidget *menu = gtk_menu_new();
	GtkWidget *item;
	GSList *group = NULL;

	GtkWidget *label = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(label), "<i>[default]</i>");
	item = gtk_radio_menu_item_new(group);
	g_signal_connect(G_OBJECT(item), "activate",
			G_CALLBACK(picture_cb), ljw);
	gtk_container_add(GTK_CONTAINER(item), label);
	gtk_widget_show_all(item);
	group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(item));
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);

	if (conf_cur_server() && conf_cur_user()) {
		GList *l;
		for (l = conf_cur_user()->pickws; l; l = l->next) {
			item = gtk_radio_menu_item_new_with_label(group, l->data);
			if (entry && entry->pickeyword &&
					strcmp(entry->pickeyword, l->data) == 0) {
				gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item),
						TRUE);
			}
			group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(item));
			g_signal_connect(G_OBJECT(item), "activate",
					G_CALLBACK(picture_cb), ljw);
			g_object_set_data(G_OBJECT(item), KEY_PICTUREKEYWORD, l->data);
			gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
			gtk_widget_show(item);
		}
	}

	return menu;
}

void
menu_update(LJWin *ljw, NetResult *result) {
	GtkWidget *menu;

	menu_update_web(ljw, result);
	gtk_label_set_text(GTK_LABEL(GTK_BIN(ljw->mloginout)->child), conf.loginok ? _("Logout") : _("Login"));
	gtk_label_set_pattern(GTK_LABEL(GTK_BIN(ljw->mloginout)->child), "_");
	menu = menu_item_from_action(ljw->factory, ACTION_JOURNAL);
	gtk_widget_set_sensitive(menu, conf.loginok);
	
	gtk_menu_item_remove_submenu(GTK_MENU_ITEM(ljw->mpicture));
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(ljw->mpicture), make_picture_menu(ljw, NULL));
}

void
menu_load_entry(LJWin *ljw, Entry *entry) {
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(
				gtk_item_factory_get_widget_by_action(ljw->factory,
					ACTION_ENTRY_SECURITY_PUBLIC+entry->security.type)), TRUE);

	gtk_menu_item_set_submenu(GTK_MENU_ITEM(ljw->mpicture),
			make_picture_menu(ljw, entry));

	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(
				gtk_item_factory_get_widget_by_action(ljw->factory,
					ACTION_ENTRY_COMMENTS_DEFAULT+entry->comments)), TRUE);

	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(
				gtk_item_factory_get_widget_by_action(ljw->factory,
					ACTION_ENTRY_BACKDATED)), entry->backdated);
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(
				gtk_item_factory_get_widget_by_action(ljw->factory,
					ACTION_ENTRY_PREFORMATTED)), entry->preformatted);
}

