/* logjam - a GTK client for LiveJournal.
 * Copyright (C) 2000-2003 Evan Martin <evan@livejournal.com>
 *
 * vim: tabstop=4 shiftwidth=4 noexpandtab :
 */

#include "config.h"

#include <gtk/gtk.h>
#include <stdlib.h> /* atoi */

#include "conf.h"
#include "util.h"
#include "meta.h"
#include "music.h"
#include "datesel.h"

struct _MetaMgr {
	GtkHBox parent;
	GtkSizeGroup *sizegroup;

	GtkWidget *date;
	GtkWidget *esubject;
	GtkWidget *cmood, *emood;
	GtkWidget *emusic, *bmusicdetect;
};

static void metamgr_init(MetaMgr *mm);
static void metamgr_class_init(GtkWidgetClass *klass);

static GtkWidgetClass *parent_class = NULL;

/* gtk stuff */
GType
metamgr_get_type(void) {
	static GType mm_type = 0;
	if (!mm_type) {
		static const GTypeInfo mm_info = {
			sizeof (GtkVBoxClass),
			NULL,
			NULL,
			(GClassInitFunc) metamgr_class_init,
			NULL,
			NULL,
			sizeof (MetaMgr),
			0,
			(GInstanceInitFunc) metamgr_init,
		};
		mm_type = g_type_register_static(GTK_TYPE_HBOX, "MetaMgr",
				&mm_info, 0);
	}
	return mm_type;
}

static GtkWidget*
labelled_box(MetaMgr *mm, const char *caption, GtkWidget *w) {
	return labelled_box_new_sg(caption, w, mm ? mm->sizegroup : NULL);
}

static GtkWidget*
make_subject(MetaMgr *mm) {
	mm->esubject = gtk_entry_new();
	return labelled_box(mm, _("_Subject:"), mm->esubject);
}

static GtkWidget*
make_date(MetaMgr *mm) {
	mm->date = datesel_new();
	return mm->date;
}
static GtkWidget*
make_subjdate(MetaMgr *mm) {
	GtkWidget *hbox;
	hbox = gtk_hbox_new(FALSE, 12);
	gtk_box_pack_start(GTK_BOX(hbox), make_subject(mm), TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), make_date(mm), FALSE, FALSE, 0);
	return hbox;
}

static GtkWidget*
make_mood(MetaMgr *mm) {
	GtkWidget *box, *combo;

	combo = gtk_combo_new();
	gtk_widget_set_size_request(combo, 20, -1);
	box = labelled_box(mm, _("_Mood:"), combo);

	mm->cmood = combo;
	mm->emood = GTK_COMBO(combo)->entry;

	return box;
}

static void
music_grab(GtkWidget *w, MetaMgr *mm) {
	char *music;
	GError *err = NULL;
	music = music_detect(&err);
	if (music) {
		gtk_entry_set_text(GTK_ENTRY(mm->emusic), music);
		g_free(music);
	} else if (err) {
		GtkWindow *toplevel;
		toplevel = GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(mm)));
		if (err->domain == MUSIC_ERROR && err->code == MUSIC_XMMS_ERROR) {
			lj_warning(toplevel,
					_("Error detecting music:\nIs XMMS running?"));
		} else {
			lj_warning(toplevel,
					_("Can't detect music: %s"), err->message);
		}
		g_error_free(err);
	}
}
static GtkWidget*
make_music(MetaMgr *mm) {
	GtkWidget *hbox, *entry, *b;

	entry = gtk_entry_new();
	hbox = labelled_box(mm, _("M_usic:"), entry);

	b = gtk_button_new_with_mnemonic(_("<- _Detect"));
	gtk_tooltips_set_tip(app.tooltips, b,
			_("Retrieve the current music from a currently-running XMMS"),
			NULL);
	g_signal_connect(G_OBJECT(b), "clicked",
			G_CALLBACK(music_grab), mm);
	gtk_box_pack_start(GTK_BOX(hbox), b, FALSE, FALSE, 0);
	
	mm->emusic = entry;
	mm->bmusicdetect = b;
	
	return hbox;
}

static GtkWidget*
make_metamain(MetaMgr *mm) {
	GtkWidget *box;
	box = gtk_vbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(box), make_subjdate(mm), FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(box), make_mood(mm),     FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(box), make_music(mm),    FALSE, FALSE, 0);
	return box;
}

static void
metamgr_init(MetaMgr *mm) {
	gtk_box_set_spacing(GTK_BOX(mm), 0);
	gtk_container_set_border_width(GTK_CONTAINER(mm), 2);

	mm->sizegroup = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);

	gtk_box_pack_start(GTK_BOX(mm), make_metamain(mm), TRUE, TRUE, 0);
}
static void
metamgr_show_all(GtkWidget *w) {
	parent_class->show_all(w);
	lj_widget_set_visible(METAMGR(w)->bmusicdetect, music_can_detect(NULL));
}

static void
metamgr_class_init(GtkWidgetClass *klass) {
	parent_class = g_type_class_peek_parent(klass);
	klass->show_all = metamgr_show_all;
}

GtkWidget*
metamgr_new(void) {
	MetaMgr *mm = METAMGR(g_object_new(metamgr_get_type(), NULL));

	return GTK_WIDGET(mm);
}

static gboolean 
meta_gtkentry_to_entry(GtkWidget *w, char **dest) {
	char *text = NULL;
	text = gtk_editable_get_chars(GTK_EDITABLE(w), 0, -1);
	if (text && text[0] == 0) {
		g_free(text);
		text = NULL;
	}
	string_replace(dest, text);
	return text != NULL;
}
static gboolean 
meta_gtkentry_from_entry(GtkWidget *w, char *src) {
	gtk_entry_set_text(GTK_ENTRY(w), src ? src : "");
	return src != NULL;
}

void 
metamgr_clear(MetaMgr *mm) {
	Entry *entry;

	entry = entry_new();
	metamgr_from_entry(mm, entry);
	entry_free(entry);
	datesel_set_tm(DATESEL(mm->date), NULL);
}

void
metamgr_to_entry(MetaMgr *mm, Entry* entry) {
	meta_gtkentry_to_entry(mm->esubject,    &entry->subject);
	meta_gtkentry_to_entry(mm->emood,       &entry->mood);
	meta_gtkentry_to_entry(mm->emusic,      &entry->music);
	datesel_get_tm(DATESEL(mm->date), &entry->time);
}

void
metamgr_from_entry(MetaMgr *mm, Entry* entry) {
	meta_gtkentry_from_entry(mm->esubject,  entry->subject);
	meta_gtkentry_from_entry(mm->emood,     entry->mood);
	meta_gtkentry_from_entry(mm->emusic,    entry->music);

	datesel_set_tm(DATESEL(mm->date), &entry->time);
}

void
metamgr_login_changed(MetaMgr *mm) {
	/* load moods. */
	{
		GList *l, *strings = NULL;
		Mood *m;
		char *text;

		text = gtk_editable_get_chars(GTK_EDITABLE(mm->emood), 0, -1);

		if (conf_cur_server()) {
			for (l = conf_cur_server()->moods; l; l = l->next) {
				m = l->data;
				strings = g_list_insert_sorted(strings, m->name,
						(GCompareFunc)g_ascii_strcasecmp);
			}
		}
		if (strings)
			gtk_combo_set_popdown_strings(GTK_COMBO(mm->cmood), strings);

		gtk_entry_set_text(GTK_ENTRY(mm->emood), text);
		g_free(text);
	}
}

void
metamgr_settings_changed(MetaMgr *mm) {
	lj_widget_set_visible(mm->bmusicdetect, music_can_detect(NULL));
}

