/* logjam - a GTK client for LiveJournal.
 * Copyright (C) 2000-2003 Evan Martin <evan@livejournal.com>
 *
 * vim: tabstop=4 shiftwidth=4 noexpandtab :
 */

#include "config.h"
#include <gtk/gtk.h>

#include <stdlib.h>

#include "protocol.h"
#include "conf.h"
#include "util.h"
#include "network.h"
#include "network-internal.h"
#include "network-throbber.h"

/* maximal response size */
#define RESPONSE_SIZE 2048

struct _NetWindow {
	GtkWindow win;
	GtkWidget *parent, *box, *label, *progress;
	NetMainloopHandle nethandle;
	gboolean showing_error;
	GtkWidget *throbber;
};

void 
net_window_show_error(NetWindow *nw, const char *fmt, ...) {
	char buf[1024];
	va_list ap;

	va_start(ap, fmt);
	g_vsnprintf(buf, 1024, fmt, ap);
	va_end(ap);

	if (nw /* XXX nsw->nw */) {
		gtk_label_set_text(GTK_LABEL(nw->label), buf);

		net_throbber_stop(NET_THROBBER(nw->throbber));
		gtk_image_set_from_stock(GTK_IMAGE(nw->throbber),
				GTK_STOCK_DIALOG_ERROR, GTK_ICON_SIZE_DIALOG);
		gtk_widget_hide(nw->progress);
		nw->showing_error = TRUE;
		gtk_main();
	} else {
		g_print(_("Error: %s\n"), buf);
	}
}

static void 
cancel_cb(NetWindow *nw) {
	if (nw->nethandle)
		net_mainloop_cancel(nw->nethandle);
	else if (nw->showing_error)
		gtk_main_quit();
}

static void
destroy_cb(GtkWidget *w, NetWindow *nw) {
	net_throbber_stop(NET_THROBBER(nw->throbber));
	g_signal_handlers_disconnect_matched(nw->parent,
			G_SIGNAL_MATCH_DATA,
			0, 0, 0, 0, nw);
}

static gboolean
delete_event_cb(GtkWidget *w, GdkEvent *e, NetWindow *nw) {
	cancel_cb(nw);
	return TRUE; /* don't destroy us. */
}

static void
reposition(NetWindow *nw, gint w, gint h) {
	gint ox, oy, px, py, x, y;

	gdk_window_get_origin(nw->parent->window, &px, &py);
	if (w == -1)
		w = GTK_WIDGET(nw)->allocation.width;
	if (h == -1)
		h = GTK_WIDGET(nw)->allocation.height;
	x = px + (nw->parent->allocation.width  - w) / 2;
	y = py + (nw->parent->allocation.height - h) / 2;

	gdk_window_get_origin(GTK_WIDGET(nw)->window, &ox, &oy);
	if (x != ox || y != oy) {
		gtk_window_move(GTK_WINDOW(nw), x, y);
	}
}

static gboolean
parent_configure_cb(GtkWidget *w, GdkEvent *e, NetWindow *nw) {
	reposition(nw, -1, -1);
	return FALSE;
}

static gboolean
configure_cb(GtkWidget *w, GdkEventConfigure *e, NetWindow *nw) {	 
	if (GTK_WIDGET(nw)->allocation.width != e->width ||
		GTK_WIDGET(nw)->allocation.height != e->height)
		reposition(nw, e->width, e->height);
	return FALSE;	 
}	 


static void
net_window_set_parent(NetWindow *nw, GtkWindow *parent) {
	nw->parent = GTK_WIDGET(parent);
	gtk_window_set_transient_for(GTK_WINDOW(nw), parent);
	/* recenter ourselves whenever our parent changes. */
	g_signal_connect(G_OBJECT(parent), "configure-event",
			G_CALLBACK(parent_configure_cb), nw);
}

void 
net_window_init(NetWindow *nw) {
	GtkWidget *frame;
	GtkWidget *vbox;
	GtkWidget *hbox, *align, *bbox, *button, *lbox;

	gtk_window_set_decorated(GTK_WINDOW(nw), FALSE);
	gtk_window_set_type_hint(GTK_WINDOW(nw), GDK_WINDOW_TYPE_HINT_DIALOG);
	gtk_window_set_modal(GTK_WINDOW(nw), TRUE);
	gtk_window_set_default_size(GTK_WINDOW(nw), 250, -1);
	g_signal_connect(G_OBJECT(nw), "destroy", G_CALLBACK(destroy_cb), nw);
	g_signal_connect(G_OBJECT(nw), "delete-event",
			G_CALLBACK(delete_event_cb), nw);
	gtk_window_set_position(GTK_WINDOW(nw), GTK_WIN_POS_CENTER_ON_PARENT);
	/* center ourselves once we're created. */
	g_signal_connect(G_OBJECT(nw), "configure-event",
			G_CALLBACK(configure_cb), nw);

	vbox = gtk_vbox_new(FALSE, 5); 
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);

	hbox = gtk_hbox_new(FALSE, 5); 

	nw->throbber = net_throbber_new();
	align = gtk_alignment_new(0.5, 0, 0, 0);
	gtk_container_add(GTK_CONTAINER(align), nw->throbber);
	gtk_box_pack_start(GTK_BOX(hbox), align, FALSE, FALSE, 0);
	net_throbber_start(NET_THROBBER(nw->throbber));

	nw->box = lbox = gtk_vbox_new(FALSE, 5);
	nw->label = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(nw->label), 0.5, 0.5);
	gtk_label_set_line_wrap(GTK_LABEL(nw->label), TRUE);
	gtk_box_pack_start(GTK_BOX(lbox), nw->label, TRUE, TRUE, 0);

	nw->progress = gtk_progress_bar_new();
	gtk_box_pack_end(GTK_BOX(lbox), nw->progress, FALSE, FALSE, 0);

	gtk_box_pack_start(GTK_BOX(hbox), lbox, TRUE, TRUE, 0);

	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);

	bbox = gtk_hbutton_box_new();
	gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_END);
	button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
	g_signal_connect_swapped(G_OBJECT(button), "clicked",
			G_CALLBACK(cancel_cb), nw);
	gtk_box_pack_end(GTK_BOX(bbox), button, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 0);

#ifndef G_OS_WIN32
	frame = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_OUT);
	gtk_container_add(GTK_CONTAINER(frame), vbox);
	gtk_widget_show_all(frame);
	gtk_container_add(GTK_CONTAINER(nw), frame);
#else
	gtk_widget_show_all(vbox);
	gtk_container_add(GTK_CONTAINER(nw), vbox);
#endif

	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
	gtk_widget_grab_default(button);
}

void
net_window_pack(NetWindow *nw, GtkWidget *contents) {
	gtk_box_pack_start(GTK_BOX(nw->box), contents, FALSE, FALSE, 0);
}

GtkWidget* 
net_window_new(GtkWindow *parent, const char *title) {
	NetWindow *nw = NET_WINDOW(g_object_new(net_window_get_type(), NULL));
	net_window_set_parent(nw, parent);
	gtk_window_set_title(GTK_WINDOW(nw), title);
	gtk_label_set_text(GTK_LABEL(nw->label), title);
	return GTK_WIDGET(nw);
}

GType
net_window_get_type(void) {
	static GType nw_type = 0;
	if (!nw_type) {
		static const GTypeInfo nw_info = {
			sizeof (GtkWindowClass),
			NULL,
			NULL,
			NULL, /*(GClassInitFunc) net_window_class_init,*/
			NULL,
			NULL,
			sizeof (NetWindow),
			0,
			(GInstanceInitFunc) net_window_init,
		};
		nw_type = g_type_register_static(GTK_TYPE_WINDOW, "NetWindow",
				&nw_info, 0);
	}
	return nw_type;
}

static void
statuscb(NetStatusType status, gpointer statusdata, gpointer data) {
	NetWindow *nw = data;
	NetStatusProgress *progress;
	float fraction = 0;

	switch (status) {
	case NET_STATUS_BEGIN:
		nw->nethandle = statusdata;
		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(nw->progress), 0);
		break;
	case NET_STATUS_PROGRESS:
		progress = (NetStatusProgress*)statusdata;
		if (progress->total > 0)
			fraction = (float)progress->current / progress->total;
		if (nw /* XXX shouldn't use this function if we have no window */) {
			gtk_widget_show(nw->progress);
			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(nw->progress),
					fraction);
		} else if (conf.options.netdump) {
			g_print(_("Progress: %.2f%%\n"), fraction*100.0);
		}
		break;
	default:
		g_warning("unhandled network status %d\n", status);
	}
}

static NetResult*
net_request_run_internal(NetRequest *request,
                         NetStatusCallback cb, gpointer data,
                         GError **err) {
	NetResult *result = NULL;
	GString *post, *res;

	post = lj_protocol_request_to_string(request);
	if (conf.options.nofork || app.cli) {
		res = net_post_blocking(post, cb, data, err);
	} else {
		res = net_post_mainloop(post, cb, data, err);
	}
	g_string_free(post, TRUE);

	if (res == NULL)
		return NULL;

	result = lj_protocol_parse_response(res->str);
	g_string_free(res, TRUE);

	if (!result || !net_result_succeeded(result)) {
		const char *msg = net_result_get(result, "errmsg");
		if (!msg)
			msg = "Unable to parse server response";
		g_set_error(err, 0, 0, _("Request failed: %s."), msg);
		if (result)
			net_result_free(result);
		return NULL;
	}

	return result;
}

NetResult*
net_window_run_request(NetWindow *nw, NetRequest *request) {
	NetResult *result = NULL;
	GError *err = NULL;

	gtk_widget_show(GTK_WIDGET(nw));
	result = net_request_run_internal(request, statuscb, nw, &err);

	if (!result) {
		if (err->code != NET_ERROR_CANCELLED)
			net_window_show_error(nw, err->message);
		g_error_free(err);
		return NULL;
	}

	return result;
}

NetResult*
net_request_run(NetRequest *request, const char *title, GtkWindow *parent) {
	GtkWidget *nw;
	NetResult *result;

	nw = net_window_new(parent, title);
	result = net_window_run_request(NET_WINDOW(nw), request);
	gtk_widget_destroy(nw);

	return result;
}

NetResult*
net_request_run_silent(NetRequest *request) {
	return net_request_run_internal(request, NULL, NULL, NULL);
}

NetResult*
net_request_run_cli(NetRequest *request) {
	NetResult *result = NULL;
	GError *err = NULL;

	result = net_request_run_internal(request, NULL, NULL, &err);
	if (!result) {
		g_print("%s\n", err->message);
		g_error_free(err);
		return NULL;
	}
	return result;
}

NetRequest*
net_request_new(const char *mode) {
	NetRequest *request = lj_protocol_request_new(mode, 
			conf_cur_user()->username, conf_cur_user()->password,
			conf.usejournal,
			conf_cur_server()->protocolversion);

	return request;
}

gboolean
net_result_succeeded(NetResult *result) {
	return lj_protocol_result_succeeded(result);
}

char*
net_result_getf(NetResult* result, const char *key, ...) {
	char buf[100];
	va_list ap;

	va_start(ap, key);
	g_vsnprintf(buf, 100, key, ap);
	va_end(ap);
	return net_result_get(result, buf); 
}

int
net_result_geti(NetResult *result, const char *key) {
	char *val;
	val = net_result_get(result, key);
	if (val)
		return atoi(val);
	return 0;
}

char*
net_result_get_prefix(NetResult *result, const char *prefix, const char *key) {
	char buf[100];
	g_snprintf(buf, 100, "%s%s", prefix, key);
	return net_result_get(result, buf);
}
