/* logjam - a GTK client for LiveJournal.
 * Copyright (C) 2000-2003 Evan Martin <evan@livejournal.com>
 *
 * vim: tabstop=4 shiftwidth=4 noexpandtab :
 */

#include "config.h"

#include <gtk/gtk.h>
#ifdef HAVE_GTKSPELL
#include <gtkspell/gtkspell.h>
#endif /* HAVE_GTKSPELL */
#include <stdlib.h> /* atoi */
#include <string.h>

#include "conf.h"
#include "security.h"
#include "util.h"
#include "spawn.h"
#include "lj.h"
#include "livejournal.h"
#include "checkfriends.h"
#include "settings.h"
#include "music.h"
#include "groupedbox.h"
#include "meta.h"
#include "remote.h"
#include "smartquotes.h"
#include "tie.h"

/* what's this?  all of these funny structures in the settings box?
 * well, instead of creating and tearing down all of these widgets, i
 * try to pull out shared functionality.
 *
 * it's loosely inspired by george lebl's pong 
 *   http://www.5z.com/jirka/pong-documentation/
 * but i've never actaully read how that works.
 *
 *
 * we have a collection of SettingsWidgets that track their name and
 * configuration option, and then it's all nicely automated through functions
 * of the form sw_*(). 
 */

typedef enum {
	SW_TOGGLE,
	SW_RADIO,
	SW_TEXT,
	SW_INTEGER,
	SW_COMBO,
	SW_SPIN_INTEGER,
	SW_CUSTOM
} SettingsWidgetType;

typedef struct _SettingsWidget SettingsWidget;
struct _SettingsWidget {
	char *name;
	void *conf;
	SettingsWidgetType type;
	char *caption;
	gpointer data; /* extra pointer, to be used as needed. */
	GtkWidget *widget;
};

static GtkWidget* cfriends_general_settings(void);
static GtkWidget* cfriends_filter_settings(void);
static GtkWidget* cfriends_indicators_settings(LJWin *ljw);

static void run_cfmask_settings_dlg(SettingsWidget *sw);

static SettingsWidget settingswidgets[] = {
	{ "ui_revertusejournal", &conf.options.revertusejournal, 
		SW_TOGGLE, N_("_Revert to primary journal after posting on a community"), NULL },
	{ "ui_defaultsecurity", &conf.defaultsecurity, 
		SW_CUSTOM, N_("Default _security on posts:") },
	{ "ui_autosave", &conf.options.autosave,
		SW_TOGGLE, N_("Automatically save _drafts (for crash recovery)") },

	{ "ui_allowmultipleinstances", &conf.options.allowmultipleinstances,
		SW_TOGGLE, N_("Allow multiple _instances of LogJam to run simultaneously") },
#ifdef HAVE_GTKSPELL
	{ "ui_spellcheck", &conf.options.usespellcheck, 
		SW_TOGGLE, N_("_Use spell check") },
	{ "ui_spell_language", &conf.spell_language,
		SW_TEXT, N_("Entry _language:") },
#endif
	{ "ui_smartquotes", &conf.options.smartquotes,
		SW_TOGGLE, N_("Automatically change _quotes to matching pairs") },
	{ "ui_font", &conf.uifont,
		SW_CUSTOM, N_("Entry display font:") },

#ifndef G_OS_WIN32
	{ "web_spawn", &conf.spawn_command, 
		SW_COMBO, NULL },

	{ "music_xmms", &conf.music_source,
		SW_RADIO, N_("Detect music using _XMMS"), 
		GINT_TO_POINTER(MUSIC_SOURCE_XMMS) },
	{ "music_command", &conf.music_source,
		SW_RADIO, N_("Detect music using _command"),
		GINT_TO_POINTER(MUSIC_SOURCE_COMMAND) },
	{ "music_command_cmdline", &conf.music_command,
		SW_TEXT, N_("Command _line:") },

	{ "net_useproxy", &conf.options.useproxy, 
		SW_TOGGLE, N_("Use _proxy server") },
	{ "net_proxy", &conf.proxy, 
		SW_TEXT, N_("UR_L:") },

	{ "net_useproxyauth", &conf.options.useproxyauth, 
		SW_TOGGLE, N_("Use proxy _authentication") },
	{ "net_proxyuser", &conf.proxyuser, 
		SW_TEXT, N_("_User:") },
	{ "net_proxypass", &conf.proxypass, 
		SW_TEXT, N_("_Password:") },
#endif

	{ "debug_netdump", &conf.options.netdump, 
		SW_TOGGLE, N_("Dump _network data on stderr") },
#ifndef G_OS_WIN32
	{ "debug_nofork", &conf.options.nofork, 
		SW_TOGGLE, N_("Don't _fork on network requests") },
#else
	{ "debug_nofork", &conf.options.nofork, 
		SW_TOGGLE, N_("Don't run network requests in a separate _thread") },
#endif


	{ "cf_autostart", &conf.options.cfautostart,
		SW_TOGGLE, N_("_Monitor friends list for new entries upon login") },
	{ "cf_usemask", &conf.options.cfusemask,
		SW_TOGGLE, N_("_Limit monitoring to groups of friends") },
	{ "cf_mask", NULL /* set at runtime to that of current user */,
		SW_CUSTOM, "" },
	{ "cf_userinterval", &conf.cfuserinterval,
		SW_SPIN_INTEGER, N_("_Check friends list every " /* X seconds */) },
	{ "cf_threshold", &conf.cfthreshold,
		SW_SPIN_INTEGER, N_("_Notify me after " /* X new entires */) },
#ifndef G_OS_WIN32
	{ "cf_docklet", &conf.options.cfdocklet,
		SW_TOGGLE, N_("System _tray indicator (For GNOME/KDE dock)") },
#endif
	{ "cf_float", &conf.options.cffloat,
		SW_TOGGLE, N_("_Floating indicator") },
	{ "cf_floatraise", &conf.options.cffloatraise,
		SW_TOGGLE, N_("_Raise floating indicator when new entries detected") },
	{ "cf_float_decorate", &conf.options.cffloat_decorate,
		SW_TOGGLE, N_("Show _titlebar on floating indicator") },
	
	{ NULL }
};

static SettingsWidget*
sw_lookup(char *name) {
	SettingsWidget *sw;
	for (sw = settingswidgets; sw->name; sw++) 
		if (strcmp(name, sw->name) == 0) return sw;
	g_error("sw_lookup failed for %s", name);
	return NULL;
}

static void
toggle_enable_cb(GtkWidget *toggle, GtkWidget *target) {
	gtk_widget_set_sensitive(target, 
			gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toggle)));
}
static void
toggle_tie_enable(GtkWidget *toggle, GtkWidget *target) {
	gtk_widget_set_sensitive(target, 
			gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toggle)));
	g_signal_connect(G_OBJECT(toggle), "toggled",
			G_CALLBACK(toggle_enable_cb), target);
}

static void
toggle_tie(SettingsWidget *sw) {
	tie_toggle(GTK_TOGGLE_BUTTON(sw->widget), (gboolean*)sw->conf);
}

static void
radio_tie_cb(GtkToggleButton *toggle, SettingsWidget *sw) {
	if (gtk_toggle_button_get_active(toggle))
		*(int*)sw->conf = GPOINTER_TO_INT(sw->data);
}
static void
radio_tie(SettingsWidget *sw) {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(sw->widget),
			*(gint*)sw->conf == GPOINTER_TO_INT(sw->data));
	g_signal_connect(G_OBJECT(sw->widget), "toggled",
			G_CALLBACK(radio_tie_cb), sw);
}

static void
text_tie(SettingsWidget *sw) {
	tie_text(GTK_ENTRY(sw->widget), (char**)sw->conf);
}

static void
integer_tie_cb(GtkEditable *e, SettingsWidget *sw) {
	char *text = gtk_editable_get_chars(e, 0, -1);
	*(int*)sw->conf = atoi(text);
	g_free(text);
}
static void
integer_tie(SettingsWidget *sw) {
	char buf[30];
	sprintf(buf, "%d", *(int*)sw->conf);
	gtk_entry_set_text(GTK_ENTRY(sw->widget), buf);
	g_signal_connect(G_OBJECT(sw->widget), "changed",
			G_CALLBACK(integer_tie_cb), sw);
}

static void
spin_integer_tie_cb(GtkSpinButton *b, SettingsWidget *sw) {
	*(gint*)sw->conf = (gint)gtk_spin_button_get_value(b);
}
static void
spin_integer_tie(SettingsWidget *sw) {
	g_signal_connect(G_OBJECT(sw->widget), "value-changed",
			G_CALLBACK(spin_integer_tie_cb), sw);
}

static void
combo_tie(SettingsWidget *sw) {
	tie_combo(GTK_COMBO(sw->widget), (char**)sw->conf);
}

static GtkWidget*
sw_make_sg(char *name, GtkSizeGroup *sg) {
	SettingsWidget *sw = sw_lookup(name);
	switch (sw->type) {
		case SW_TOGGLE:
			sw->widget = gtk_check_button_new_with_mnemonic(_(sw->caption));
			toggle_tie(sw);
			return sw->widget;
		case SW_RADIO:
			sw->widget = gtk_radio_button_new_with_mnemonic(NULL,
					_(sw->caption));
			radio_tie(sw);
			return sw->widget;
		case SW_TEXT:
			sw->widget = gtk_entry_new();
			text_tie(sw);
			return labelled_box_new_sg(_(sw->caption), sw->widget, sg);
		case SW_INTEGER:
			sw->widget = gtk_entry_new();
			gtk_entry_set_width_chars(GTK_ENTRY(sw->widget), 4);
			integer_tie(sw);
			return labelled_box_new(_(sw->caption), sw->widget);
		case SW_SPIN_INTEGER:
			{
				gdouble default_value = *(gint*)sw->conf;
				GtkObject *adj = gtk_adjustment_new(default_value,
						/* client code should override these */
						G_MINDOUBLE, G_MAXDOUBLE, 1.0, 1.0, 1.0);
				sw->widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1, 1);
			}
			spin_integer_tie(sw);
			return labelled_box_new_expand(_(sw->caption), sw->widget, FALSE);
		case SW_COMBO:
			sw->widget = gtk_combo_new();
			combo_tie(sw);
			return labelled_box_new(_(sw->caption), sw->widget);
		case SW_CUSTOM:
			if (sw->caption && sw->widget)
				return labelled_box_new(_(sw->caption), sw->widget);
			break;
		default:
			break;
	}
	return NULL;
}
static GtkWidget*
sw_make(char *name) {
	return sw_make_sg(name, NULL);
}

static void
sec_changed_cb(SecMgr *sm, SettingsWidget *sw) {
	secmgr_security_get(sm, (Security*)sw->conf);
}

static void
run_fontsel_settings_dlg(SettingsWidget *sw) {
	GtkWidget *dlg;
	const gchar *newfont;
	gchar *oldfont;

	dlg = gtk_font_selection_dialog_new(_("Select font"));
	gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(dlg),
			gtk_label_get_text(GTK_LABEL(sw->widget))); 
	
	if (gtk_dialog_run(GTK_DIALOG(dlg)) == GTK_RESPONSE_OK) {
		gtk_label_set_text(GTK_LABEL(sw->widget), 
				gtk_font_selection_dialog_get_font_name(
						GTK_FONT_SELECTION_DIALOG(dlg)));
	}


	newfont = gtk_label_get_text(GTK_LABEL(sw->widget));
	oldfont = pango_font_description_to_string(
			pango_context_get_font_description(
				gtk_widget_get_pango_context(GTK_WIDGET(sw->data))));

	if (newfont && g_ascii_strcasecmp(oldfont, newfont) != 0) {
		string_replace(sw->conf, g_strdup(newfont));
		lj_widget_set_font(sw->widget, newfont);
		lj_widget_set_font(sw->data, newfont);
	}
	g_free(oldfont);
	
	gtk_widget_destroy(dlg); 
}

static GtkWidget*
uisettings(LJWin *ljw) {
	SettingsWidget *sw;
	GtkWidget *vbox, *hbox, *button;
	char *fontname = NULL;
	GtkWidget *post, *misc;

	vbox = gtk_vbox_new(FALSE, 18);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);

	post = groupedbox_new_with_text(_("Post Options"));
	gtk_box_pack_start(GTK_BOX(vbox), post, FALSE, FALSE, 0);

	groupedbox_pack(GROUPEDBOX(post),
			sw_make("ui_revertusejournal"), FALSE);

	sw = sw_lookup("ui_defaultsecurity");
	sw->widget = secmgr_new(FALSE);
	g_signal_connect(G_OBJECT(sw->widget), "changed",
			G_CALLBACK(sec_changed_cb), sw);
	secmgr_security_set(SECMGR(sw->widget), (Security*)sw->conf);
	groupedbox_pack(GROUPEDBOX(post), sw_make("ui_defaultsecurity"), FALSE);

	groupedbox_pack(GROUPEDBOX(post), sw_make("ui_autosave"), FALSE);

	misc = groupedbox_new_with_text(_("Behavior"));
	gtk_box_pack_start(GTK_BOX(vbox), misc, FALSE, FALSE, 0);

	groupedbox_pack(GROUPEDBOX(misc),
			sw_make("ui_allowmultipleinstances"), FALSE);

#ifdef HAVE_GTKSPELL
	{
		GtkWidget *toggle = sw_make("ui_spellcheck");
		GtkWidget *label, *box;

		hbox = sw_make("ui_spell_language");
		label = gtk_label_new("        ");
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
		gtk_box_reorder_child(GTK_BOX(hbox), label, 0);

		sw = sw_lookup("ui_spell_language");
		gtk_entry_set_width_chars(GTK_ENTRY(sw->widget), 5);

		box = gtk_vbox_new(FALSE, 3);
		gtk_box_pack_start(GTK_BOX(box), toggle, FALSE, FALSE, 0);
		gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 0);

		groupedbox_pack(GROUPEDBOX(misc), box, FALSE);
		toggle_tie_enable(toggle, hbox);
	}
#endif

	groupedbox_pack(GROUPEDBOX(misc),
			sw_make("ui_smartquotes"), FALSE);
	
	sw = sw_lookup("ui_font");
	if (conf.uifont == NULL) {
		fontname = pango_font_description_to_string(
				pango_context_get_font_description(
					gtk_widget_get_pango_context(GTK_WIDGET(ljw->eentry))));
		sw->widget = gtk_label_new(fontname ? fontname : _("[gtk default]"));
		if (fontname)
			lj_widget_set_font(sw->widget, fontname);
		g_free(fontname);
	} else {
		sw->widget = gtk_label_new(conf.uifont);
		lj_widget_set_font(sw->widget, conf.uifont);
	}
	button = gtk_button_new_from_stock(GTK_STOCK_SELECT_FONT);
	sw->data = ljw->eentry;

	hbox = labelled_box_new_expand(_(sw->caption), button, FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), sw->widget, TRUE, TRUE, 0);
	gtk_box_reorder_child(GTK_BOX(hbox), sw->widget, 1);

	groupedbox_pack(GROUPEDBOX(misc), hbox, FALSE);
	g_signal_connect_swapped(G_OBJECT(button), "clicked",
			G_CALLBACK(run_fontsel_settings_dlg), sw);

	return vbox;
}

#ifndef G_OS_WIN32
static GtkWidget*
proxyuserpass() {
	GtkWidget *vbox;
	GtkSizeGroup *sg;
	SettingsWidget *sw;

	vbox = gtk_vbox_new(FALSE, 6);

	sg = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);

	gtk_box_pack_start(GTK_BOX(vbox),
			sw_make_sg("net_proxyuser", sg), FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox),
			sw_make_sg("net_proxypass", sg), FALSE, FALSE, 0);
	sw = sw_lookup("net_proxypass");
	gtk_entry_set_visibility(GTK_ENTRY(sw->widget), FALSE);

	return vbox;
}

static void
music_diagnose(GtkWidget *button) {
	GtkWindow *dlg = GTK_WINDOW(gtk_widget_get_toplevel(button));
	GError *err = NULL;
	char *music, *result;

	if (!music_can_detect(&err)) {
		if (err->domain == MUSIC_ERROR && err->code == MUSIC_OPEN_ERROR) {
			lj_warning(dlg, _("%s\n\n"
						"If LogJam was unable to find libxmms.so.1, "
						"please verify you have XMMS installed."),
					err->message);
		} else {
			lj_warning(dlg, "%s", err->message);
		}
		g_error_free(err);
		return;
	}

	music = music_detect(&err);
	if (!music) {
		if (err->domain == MUSIC_ERROR && err->code == MUSIC_XMMS_ERROR) {
			lj_warning(dlg, _("Error detecting music:\n%s\n\nIs XMMS running?"), err->message);
		} else {
			lj_warning(dlg, _("Can't detect music: %s"), err->message);
		}
		g_error_free(err);
		return;
	}
	result = g_strdup_printf(_("Music detection succeeded.\n\nCurrent music:\n%s"), music);
	lj_messagebox(dlg, _("Music Detection"), result);
	g_free(music);
	g_free(result);
}

static GtkWidget*
musicsettingsgroup(void) {
	GtkWidget *group, *opt1, *opt2, *vbox, *hbox, *e;

	group = groupedbox_new_with_text(_("Music"));

	opt1 = sw_make("music_xmms");
	groupedbox_pack(GROUPEDBOX(group), opt1, FALSE);
#if XMMS_TYPE == XMMS_TYPE_NONE
	gtk_widget_set_sensitive(opt1, FALSE);
#endif

	opt2 = sw_make("music_command");
	gtk_radio_button_set_group(GTK_RADIO_BUTTON(opt2),
			gtk_radio_button_get_group(GTK_RADIO_BUTTON(opt1)));
	/* we have to set the widgets active after they've been made,
	 * because the group was created afterwards. */
	if (conf.music_source == MUSIC_SOURCE_XMMS)
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(opt1), TRUE);
	else
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(opt2), TRUE);

	e = sw_make("music_command_cmdline");
	toggle_tie_enable(opt2, e);

	vbox = gtk_vbox_new(FALSE, 3);
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new("        "), FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), e, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), opt2, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

	groupedbox_pack(GROUPEDBOX(group), vbox, FALSE);
	return group;
}

static GtkWidget*
musicdiagnosegroup(void) {
	GtkWidget *group, *button, *hbox;

	group = groupedbox_new_with_text(_("Diagnostic"));

	button = gtk_button_new_with_mnemonic(_("_Diagnose"));
	g_signal_connect(G_OBJECT(button), "clicked",
			G_CALLBACK(music_diagnose), NULL);
	hbox = labelled_box_new_expand(_("Diagnose problems detecting music:"),
			button, FALSE);
	groupedbox_pack(GROUPEDBOX(group), hbox, FALSE);

	return group;
}

static GtkWidget*
proxysettings(void) {
	GtkWidget *group, *wbox, *hbox, *cb, *table, *authgroup;

	group = groupedbox_new_with_text(_("Proxy"));

	cb = sw_make("net_useproxy");
	groupedbox_pack(GROUPEDBOX(group), cb, FALSE);

	wbox = gtk_vbox_new(FALSE, 6);
	toggle_tie_enable(cb, wbox);
	groupedbox_pack(GROUPEDBOX(group), wbox, FALSE);

	hbox = sw_make("net_proxy");
	gtk_box_pack_start(GTK_BOX(wbox), hbox, FALSE, FALSE, 0);

	cb = sw_make("net_useproxyauth");
	gtk_box_pack_start(GTK_BOX(wbox), cb, FALSE, FALSE, 0);

	table = proxyuserpass();
	authgroup = groupedbox_new();
	groupedbox_pack(GROUPEDBOX(authgroup), table, FALSE);
	gtk_box_pack_start(GTK_BOX(wbox), authgroup, FALSE, FALSE, 0);

	toggle_tie_enable(cb, table);

	return group;
}

static GtkWidget*
websettings(void) {
	GtkWidget *group;
	GtkWidget *combo, *label;
	GList *list = NULL;
	int i;

	group = groupedbox_new_with_text(_("Web Browser"));

	label = gtk_label_new(
			_("This command is evaluated by /bin/sh.\n"
			"Use %s in the command line in the place of the URL.\n"
			"You can use %s more than once."));
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
	groupedbox_pack(GROUPEDBOX(group), label, FALSE);

	for (i = 0; spawn_commands[i] != NULL; i++) {
		list = g_list_append(list, spawn_commands[i]);
	}

	groupedbox_pack(GROUPEDBOX(group), sw_make("web_spawn"), FALSE);

	combo = sw_lookup("web_spawn")->widget;
	if (conf.spawn_command) {
		char *origspawn;
		/* setting the popdown strings clobbers the entry text,
		 * which auto-updates conf.spawn_command.  so we back
		 * it up before we set popdown_strings and restore it
		 * afterwards. */
		origspawn = g_strdup(conf.spawn_command);
		gtk_combo_set_popdown_strings(GTK_COMBO(combo), list);
		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(combo)->entry), origspawn);
		g_free(origspawn);
	} else {
		gtk_combo_set_popdown_strings(GTK_COMBO(combo), list);
	}

	return group;
}

static GtkWidget*
musicsettings(void) {
	GtkWidget *vbox;

	vbox = gtk_vbox_new(FALSE, 18);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
	gtk_box_pack_start(GTK_BOX(vbox), musicsettingsgroup(), FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), musicdiagnosegroup(), FALSE, FALSE, 0);
	return vbox;
}

static GtkWidget*
systemsettings(void) {
	GtkWidget *vbox;

	vbox = gtk_vbox_new(FALSE, 18);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
	gtk_box_pack_start(GTK_BOX(vbox), websettings(), FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), proxysettings(), FALSE, FALSE, 0);
	return vbox;
}
#endif /* G_OS_WIN32 */

static GtkWidget*
debugsettings(GtkWidget *dlg) {
	GtkWidget *vbox, *gb;

	vbox = gtk_vbox_new(FALSE, 18);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);

	gb = groupedbox_new_with_text(_("Network"));
	gtk_box_pack_start(GTK_BOX(vbox), gb, FALSE, FALSE, 0);

	groupedbox_pack(GROUPEDBOX(gb), sw_make("debug_netdump"), FALSE);
	groupedbox_pack(GROUPEDBOX(gb), sw_make("debug_nofork"), FALSE);

	return vbox;
}

static GtkWidget*
cfriendssettings(LJWin *ljw) {
	GtkWidget *vbox;

	vbox = gtk_vbox_new(FALSE, 5);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);

	/* general */
	gtk_box_pack_start(GTK_BOX(vbox), cfriends_general_settings(),
			FALSE, FALSE, 0);

	/* spacer */
	gtk_box_pack_start(GTK_BOX(vbox), gtk_label_new(""), FALSE, FALSE, 0);
	
	/* indicators */
	gtk_box_pack_start(GTK_BOX(vbox), cfriends_indicators_settings(ljw),
			FALSE, FALSE, 0);

	return vbox;
}

static GtkWidget*
cfriends_general_settings() {
	GtkWidget *general, *b, *w;
	SettingsWidget *sw;
	
	general = groupedbox_new_with_text(_("General"));

	groupedbox_pack(GROUPEDBOX(general), sw_make("cf_autostart"), FALSE);

	b  = sw_make("cf_threshold");
	sw = sw_lookup("cf_threshold");
	w  = sw->widget;
	lj_spin_button_set(GTK_SPIN_BUTTON(w), TRUE /* numeric */,
			1.0, 7.0 /* range */, 1.0, 1.0 /*increments */, 0 /* digits */);
	gtk_box_pack_start(GTK_BOX(b),
			gtk_label_new(_("new entries by my friends")), FALSE, FALSE, 5);
	/* XXX: ugly, because it defaults to "1 entries". Yuck. */
	groupedbox_pack(GROUPEDBOX(general), b, FALSE);

	b  = sw_make("cf_userinterval");
	sw = sw_lookup("cf_userinterval");
	w  = sw->widget;
	lj_spin_button_set(GTK_SPIN_BUTTON(w), TRUE /*numeric*/,
			45.0, 3600.0 /*range*/, 1.0, 10.0 /*increments*/, 0 /*digits*/);
	gtk_box_pack_start(GTK_BOX(b), gtk_label_new(_("seconds")),
			FALSE, FALSE, 5);
	groupedbox_pack(GROUPEDBOX(general), b, FALSE);

	/* filter */
	groupedbox_pack(GROUPEDBOX(general),
			cfriends_filter_settings(), FALSE);
	
	return general;
}

static GtkWidget*
cfriends_filter_settings() {
	GtkWidget *filtervbox, *maskhbox, *l;
	SettingsWidget *sw;
	gboolean fgsafe = conf.loginok &&
		conf_cur_user() && conf_cur_user()->friendgroups;
	
	filtervbox = gtk_vbox_new(FALSE, 5);

	maskhbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(maskhbox), 
			sw_make("cf_usemask"), FALSE, FALSE, 0);
	sw = sw_lookup("cf_mask");
	sw->conf = &(conf_cur_user()->cfmask);
	sw->widget = gtk_button_new_with_label(_("Choose filter"));
	sw->data = (fgsafe ? &conf_cur_user()->cfmask : NULL);
	gtk_box_pack_start(GTK_BOX(maskhbox), GTK_WIDGET(sw->widget), FALSE, FALSE, 0);
	gtk_widget_set_sensitive(sw->widget, fgsafe);
	g_signal_connect_swapped(G_OBJECT(sw->widget), "clicked",
			G_CALLBACK(run_cfmask_settings_dlg), sw);
	l = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(l),
			_("<small>The filter chosen here only affects the "
				"current user.</small>"));
	gtk_label_set_line_wrap(GTK_LABEL(l), TRUE);
	gtk_label_set_justify(GTK_LABEL(l), GTK_JUSTIFY_LEFT);

	gtk_box_pack_start(GTK_BOX(filtervbox), maskhbox, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(filtervbox), l,        FALSE, FALSE, 0);

	return filtervbox;
}

#ifndef G_OS_WIN32
static void
dock_change_cb(GtkWidget *w, LJWin *ljw) {
	cf_update_dock(ljw->cfmgr, GTK_WINDOW(ljw));
}
#endif /* G_OS_WIN32 */
static void
float_change_cb(GtkWidget *w, LJWin *ljw) {
	cf_update_float(ljw->cfmgr, GTK_WINDOW(ljw));
}

static GtkWidget*
cfriends_indicators_settings(LJWin *ljw) {
	GtkWidget *indicators, *floaters, *b, *w;
	SettingsWidget *sw;
	
	indicators = groupedbox_new_with_text(_("Indicators"));

#ifndef G_OS_WIN32
	b  = sw_make("cf_docklet");
	g_signal_connect(G_OBJECT(b), "toggled",
			G_CALLBACK(dock_change_cb), ljw);
	groupedbox_pack(GROUPEDBOX(indicators), b, FALSE);
#endif /* G_OS_WIN32 */

	b  = sw_make("cf_float");
	g_signal_connect(G_OBJECT(b), "toggled",
			G_CALLBACK(float_change_cb), ljw);
	sw = sw_lookup("cf_float");
	w  = sw->widget;
	groupedbox_pack(GROUPEDBOX(indicators), b, FALSE);

	floaters = groupedbox_new();
	groupedbox_pack(GROUPEDBOX(indicators), floaters, FALSE);
	
	groupedbox_pack(GROUPEDBOX(floaters), sw_make("cf_floatraise"), FALSE);
	b = sw_make("cf_float_decorate");
	g_signal_connect(G_OBJECT(b), "toggled",
			G_CALLBACK(cf_float_decorate_refresh), ljw);
	groupedbox_pack(GROUPEDBOX(floaters), b, FALSE);

	toggle_tie_enable(w, floaters);

	return indicators;
}

static gboolean
run_settings_dialog(LJWin *ljw, SettingsPage page) {
	GtkWidget *dlg, *nb;

	dlg = gtk_dialog_new_with_buttons(_("Preferences"),
			GTK_WINDOW(ljw), GTK_DIALOG_MODAL,
			GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
			NULL);
	gtk_dialog_set_has_separator(GTK_DIALOG(dlg), FALSE);

	/* the order of notebook pages created here should match the
	 * SettingsPage enum in settings.h */
	nb = gtk_notebook_new();
	gtk_notebook_append_page(GTK_NOTEBOOK(nb), 
			uisettings(ljw), gtk_label_new_with_mnemonic(_("User Interface")));
#ifndef G_OS_WIN32
	gtk_notebook_append_page(GTK_NOTEBOOK(nb), 
			musicsettings(), gtk_label_new_with_mnemonic(_("Music")));
	gtk_notebook_append_page(GTK_NOTEBOOK(nb), 
			systemsettings(), gtk_label_new_with_mnemonic(_("System")));
#endif /* G_OS_WIN32 */
	gtk_notebook_append_page(GTK_NOTEBOOK(nb), 
			cfriendssettings(ljw), gtk_label_new_with_mnemonic(_("Check Friends")));
	gtk_notebook_append_page(GTK_NOTEBOOK(nb), 
			debugsettings(dlg), gtk_label_new_with_mnemonic(_("Debug")));

	lj_dialog_set_contents_container(GTK_DIALOG(dlg), nb);
	gtk_notebook_set_current_page(GTK_NOTEBOOK(nb), page);

	gtk_dialog_run(GTK_DIALOG(dlg));
	gtk_widget_destroy(dlg);
	return TRUE;
}

void
settings_run(LJWin *ljw, SettingsPage initialpage) {
#ifdef HAVE_GTKSPELL
	gboolean hadspell    = conf.options.usespellcheck;
#endif
	gboolean hadautosave = conf.options.autosave;
	gboolean hadquotes   = conf.options.smartquotes;

	if (!run_settings_dialog(ljw, initialpage))
		return;
	
	g_slist_foreach(app.secmgr_list, (GFunc)secmgr_security_set_force,
			&conf.defaultsecurity);

	cf_threshold_normalize(&conf.cfthreshold);

	if (!hadautosave && conf.options.autosave)
		lj_autosave_init(ljw);
	if (hadautosave && !conf.options.autosave)
		lj_autosave_stop(ljw);

	if (conf.options.allowmultipleinstances) {
		GError *err = NULL;
		if (!logjam_remote_stop_listening(app.remote, &err)) {
			lj_warning(GTK_WINDOW(ljw),
					_("Error stopping remote listener: %s."), err->message);
			g_error_free(err);
		}
	}
	if (!conf.options.allowmultipleinstances) {
		GError *err = NULL;
		if (!logjam_remote_listen(app.remote, &err) && err) {
			lj_warning(GTK_WINDOW(ljw),
					_("Error starting remote listener: %s."), err->message);
			g_error_free(err);
		}
	}

	if (conf.options.smartquotes && !hadquotes)
		smartquotes_attach(logjam_doc_get_text_buffer(ljw->doc));
	else if (!conf.options.smartquotes && hadquotes)
		smartquotes_detach(logjam_doc_get_text_buffer(ljw->doc));

	metamgr_settings_changed(METAMGR(ljw->metamgr));

#ifdef HAVE_GTKSPELL
	if (conf.options.usespellcheck != hadspell) {
		GtkSpell *spell;
		if (conf.options.usespellcheck) {
			GError *err = NULL;
			if (gtkspell_new_attach(GTK_TEXT_VIEW(ljw->eentry), conf.spell_language , &err) == NULL) {
				lj_warning(GTK_WINDOW(ljw),
						_("GtkSpell error: %s"), err->message);
				conf.options.usespellcheck = FALSE;
				g_error_free(err);
			}
		} else {
			spell = gtkspell_get_from_text_view(GTK_TEXT_VIEW(ljw->eentry));
			if (spell)
				gtkspell_detach(spell);
		}
	}
#endif
}

static void
run_cfmask_settings_dlg(SettingsWidget *sw) {
	GtkWindow *window = GTK_WINDOW(gtk_widget_get_toplevel(sw->widget));
	*(int*)sw->data = custom_security_dlg_run(window, *(int*)sw->data, NULL);
}

